<?php
/* Logaholic Web Analytics software             Copyright(c) 2005-2011 Logaholic B.V.
 *                                                               All rights Reserved.
 * This code is subject to the Logaholic license. Unauthorized copying is prohibited.
 * support@logaholic.com                         http://www.logaholic.com/License.txt
*/ 

// core util function consolidation; aids in customization
require_once "core_factory.php";

//This file includes the software version and db version numbers
require_once("includes/version.php"); 

//error reporting
if (LOGAHOLIC_VERSION_STATUS=="release") { 
    error_reporting(E_ALL & ~E_NOTICE);    
} else {
    error_reporting(E_ALL);
}

@ini_set('session.cookie_domain',$_SERVER['HTTP_HOST']);

Logaholic_sessionStart();

if(!isset($_SESSION['logaholic_dir']) || $_SESSION['logaholic_dir'] !== logaholic_dir()){
	foreach ($_SESSION as $k => $v) {
		unset($_SESSION[$k]);
	}
}
$_SESSION['logaholic_dir'] = logaholic_dir();

// If the debug parameter was passed in, then set it sticky in the session (either on or off).

if (isset($_GET["debug"])) {
    if ($_GET["debug"]=="off") {
        $_SESSION["debug"] = "";
    } else {
	    $_SESSION["debug"] = $_GET["debug"];
    }
}
if (isset($_SESSION["debug"])) {
	$debug = $_SESSION["debug"];
}

global $lang;
$lang_has_changed = Logaholic_hasLangChanged();
$lang = Logaholic_setLang();

/**
* Default Timezone. We need to set a default or we get warnings
*/
if (function_exists("date_default_timezone_get")) {
    $system_timezone = @date_default_timezone_get();
    if ($system_timezone!="") {
        date_default_timezone_set("$system_timezone");    
    }
}    

# All database errors will be raised as errors.  No need for special handling.
require_once("components/adodb/adodb-errorhandler.inc.php");  
# Use adodb for database work.
require_once("components/adodb/adodb.inc.php");

require_once("markup.php");
if (LOGAHOLIC_BASE_EDITION != "cPanel Edition") {
	@include_once("user_settings.php");  // This is an optional file that can have several settings in it for user settings.
	// -- Possible user_settings value --
	// $do_today_as_yesterday - set this to true to do the today page as if it was midnight minus 1 second of yesterday.
	// functions: do_extra_visual_mode_url_parse and do_extra_visual_mode_page_parse that are called during visual page mode parsing to help site-specific code
}

// Include siteprofile class
include_once "includes/siteprofile.php";
// Turn on error handling...
@include_once("errorhandling.inc.php");
$errorHandler = @(new clsErrorHandle());


if (!$running_from_command_line) {
	foreach($_GET as $k => $v) {
		if($k !== 'saveDashboard' && $k !== 'logdata' && $k !== 'serverdata'){
			SecurityCheck($v);
		}
	}
	
	foreach($_POST as $k => $v) {
		if($k !== 'saveDashboard' && $k !== 'logdata' && $k !== 'serverdata'){
			SecurityCheck($v);
		}
	}
}

if (isset($_REQUEST["trafficsource"])) { $_SESSION["trafficsource"] = $_REQUEST["trafficsource"]; }

// Pull in some default values from the URL.
$conf = isset($conf) ? $conf : @$_REQUEST["conf"];
$from = @$_REQUEST["from"];
$to = @$_REQUEST["to"];
$limit = @$_REQUEST["limit"];
$editconf=@$_REQUEST["editconf"];

// Visitor Identification Methods
define("VIDM_IPADDRESS", 1);  	// IP Address only
define("VIDM_IPPLUSAGENT", 2);  // IP address + user agent
define("VIDM_COOKIE", 3);  // User cookie - we don't support this yet!

$vidmethods[VIDM_IPADDRESS] = "IP Address Only";
$vidmethods[VIDM_IPPLUSAGENT] = "IP Address and User Agent";
$vidmethods[VIDM_COOKIE] = "Cookie based"; 

# What databases do we support?
$supported_databases = array("mysql" => "MySQL (4.1 or higher recommended)"); 

$tableheaderfontcolor = "black";  // Temporary (?) to get rid of warnings.	

$validUserRequired = false;
$loginSystemExists = false;

# Log in to the database.  
if (!isset($mysqlsave)) {
	
	@include "files/global.php";
	
	if(!isset($mysqlprefix)) { $mysqlprefix = ""; }
	
	/**
	 * Database Table Constants - these constants
	 * hold the names of all the database tables used
	 * in the script.
	 */
	include_once("includes/table_definitions.php");
	
	# check if we have variables from global.php
	if (!isset($mysqlserver)) {
		if (_LOGAHOLIC_EDITION == 4 || LOGAHOLIC_BASE_EDITION == "cPanel Edition") {
			die("Global database variables aren't set.  Please contact your hosting provider.");
		} else {
			die("Global database variables aren't set.  Please run <a href=install.php>installation</a>.");
		}
	}
	
	if (!isset($databasedriver) || (!$databasedriver)) {
		$databasedriver = "mysql";
	}
  
	if ($databasedriver == "mysql") {
        if (function_exists("mysqli_connect")) {
			$db = ADONewConnection("mysqli");
		} else {
			$db = ADONewConnection("mysql");
		}
	} else if ($databasedriver = "sqlite") {
		if (phpversion() >= "5.0.0") {
			$db = ADONewConnection("pdo_sqlite");
		} else {
		  /**
		  * PhpEd needs some help figuring out the class type, since ADONewConnection returns an abstraction.
		  * @var ADOConnection 
		  */ 
			$db = ADONewConnection("sqlite");
		}
	} else {
		die("Database driver: ".$databasedriver." not supported.");
	}
    
    @$db->Connect($mysqlserver,$mysqluname,$mysqlpw, $DatabaseName);
    if (!$db->IsConnected()) {		
		die("Couldn't connect to database " . $db->ErrorMsg());		
	} else {
		$db->SetCharSet('utf8');
	} 
	
	$vnum = $db->ServerInfo();
	$vnumfull = $vnum["version"]; 
    $vnum = substr($vnum["version"],0,3);    
		
	if (($databasedriver == "mysql") && ($vnum < "4.1")) {
		$supports_correlated_subselects = false;
	} else {
		$supports_correlated_subselects = true;
	}	
	
	# If the user login system exists, then try and access it.   (The @ supressess the error if it doesn't exist).
	if (!@$running_from_command_line) {
        @include_once("user_login/login.inc.php");
		if (!$validUserRequired) {
			$session->isAdminFlag=1;
		}
		
        # if we are not an admin, we are not allowed to do debug mode 
        if (($validUserRequired) && (!$session->isAdmin())) {
            if ($debug) {
                $_SESSION["debug"] = "";
                echo "<font color=red>Debug mode is only allowed for admin users!</font>";
                exit();    
            }
        }		
	}
	
	# If the debug parameter was passed in, then set it sticky in the session (either on or off).
	# NOTE ** This session access needs to be *after* the user_login include, because that
	# actually starts the session.
	
	# Passing debug=0 or debug=off will turn off debug mode.       
	if (isset($_SESSION["debug"]) && ($_SESSION["debug"] != 0) && (strtolower($_SESSION["debug"]) != "off")) {
		$debug = $_SESSION["debug"];
		$db->debug = true;
	}
	
	
	# Needed for version_check
	if(!isset($_SESSION["cur_dbver"])){
		$_SESSION["cur_dbver"] = getGlobalSetting("DB_MetadataVersion", 0.00);
	}

	if(!isset($_SESSION["user_dbver"])){
		$_SESSION["user_dbver"] = getGlobalSetting("DB_UserDataVersion", 0.00);
	}
	
	$cur_dbver = $_SESSION["cur_dbver"];
	$user_dbver = $_SESSION["user_dbver"];
	
	# Check to see the version number of the database schema.  If it needs to be updated then load that file
	if ($cur_dbver < CURRENT_DB_VERSION) {
		logDebugMessage("Metadata version update needed.  Current: $cur_dbver, Required: ".CURRENT_DB_VERSION); 
		# We should update.  For the current build, required and minimum versions are the same - so just *do* the update.
		include_once "version_check.php";
	}
	
	# Check to see the version number of the user database schema.    
	if (($loginSystemExists ==true) && ($user_dbver < USER_DB_VERSION)) {
        include_once "version_check.php";            
    }
    
    # Check if we should load IP encoding for german market
    if (@$ipencoding==true) {
        include_once("includes/spqr.php");
    }     
}

# FROM HERE WE GOT $db!
 
# Does this release uses the report store? Enter true or false via API
if (@_LOGAHOLIC_EDITION == 4 || LOGAHOLIC_BASE_EDITION == "cPanel Edition") {
	$default_store = false;
}else{
	$default_store = true;
}

if(!isset($_SESSION["report_store"])){
	$_SESSION["report_store"] = getGlobalSetting("logaholic_report_store", $default_store);
}
define("LOGAHOLIC_REPORT_STORE", $_SESSION["report_store"]);

$cnames = CountryNames();
require_once "includes/dateselector.php";
logDebugMessage(_LOADING_PAGE." ".@$_SERVER['SCRIPT_NAME']);
// Are we processing a login request?
if ($validUserRequired) {
	require_once "user_login/process.php";
	/* Initialize process */
	$process = new Process;
}

// Setup template
include_once "templates/template.php";
include_once "templates/template_v3.php";
$template = new Template_v3();


# this adds json support to PHP versions that don't have it
if(!function_exists('json_encode')) {
    include_once("components/json/JSON.php");
    $GLOBALS['JSON_OBJECT'] = new Services_JSON();
    function json_encode($value)
    {
        return $GLOBALS['JSON_OBJECT']->encode($value); 
    }
    
    function json_decode($value, $assoc = false)
    {
		if ($assoc == true) {
			return stdToAssoc($GLOBALS['JSON_OBJECT']->decode($value));
		} else {
			return $GLOBALS['JSON_OBJECT']->decode($value);
		}
    }
}

if(!empty($conf) && empty($_REQUEST['new'])){
	#Checks if setted conf exists.
	$checkprofiles = $db->Execute("Select profilename from ".TBL_PROFILES." WHERE profilename='{$conf}'");
	$prof = $checkprofiles->FetchRow();
	if(empty($prof['profilename'])){
		# If no profile is set, redirect to the profiles page.
		echo $template->NoAccess("This profile does not exist!");
		exit;
	}
}
# this adds class based reports
if (file_exists("includes/report.php")) {
    include_once "includes/report.php";
}

if ($lang_has_changed==true) {
	deleteProfileData($conf, $conf.".cache\_%");
}

if(isset($dont_close_session) == false) {
	if (isset($profile) && $profile->profileloaded && ($profile->structure_version < CURRENT_PROFILE_STRUCTURE_VERSION)) {
		# don't close the session we need to upgrade the profile structure so we need a session
	} else {
		session_write_close();
	}
}

function getmicrotime() {
	list($usec, $sec) = explode(" ", microtime()); 
	return ((float)$usec + (float)$sec); 
}


function CountryNames() {
    //set up country names;
    $cnames[_UNKNOWN]=_UNKNOWN;
    $cnames["A1"]=_A1;
    $cnames["A2"]=_A2;
    $cnames["AD"]=_AD;
    $cnames["AE"]=_AE;
    $cnames["AF"]=_AF;
    $cnames["AG"]=_AG;
    $cnames["AI"]=_AI;
    $cnames["AL"]=_AL;
    $cnames["AM"]=_AM;
    $cnames["AN"]=_AN;
    $cnames["AO"]=_AO;
    $cnames["AP"]=_AP;
    $cnames["AQ"]=_AQ;
    $cnames["AR"]=_AR;
    $cnames["AS"]=_AS;
    $cnames["AT"]=_AT;
    $cnames["AU"]=_AU;
    $cnames["AW"]=_AW;
    $cnames["AX"]=_AX;
    $cnames["AZ"]=_AZ;
    $cnames["BA"]=_BA;
    $cnames["BB"]=_BB;
    $cnames["BD"]=_BD;
    $cnames["BE"]=_BE;
    $cnames["BF"]=_BF;
    $cnames["BG"]=_BG;
    $cnames["BH"]=_BH;
    $cnames["BI"]=_BI;
    $cnames["BJ"]=_BJ;
    $cnames["BM"]=_BM;
    $cnames["BN"]=_BN;
    $cnames["BO"]=_BO;
    $cnames["BR"]=_BR;
    $cnames["BS"]=_BS;
    $cnames["BT"]=_BT;
    $cnames["BV"]=_BV;
    $cnames["BW"]=_BW;
    $cnames["BY"]=_BY;
    $cnames["BZ"]=_BZ;
    $cnames["CA"]=_CA;
    $cnames["CC"]=_CC;
    $cnames["CD"]=_CD;
    $cnames["CF"]=_CF;
    $cnames["CG"]=_CG;
    $cnames["CH"]=_CH;
    $cnames["CI"]=_CI;
    $cnames["CK"]=_CK;
    $cnames["CL"]=_CL;
    $cnames["CM"]=_CM;
    $cnames["CN"]=_CN;
    $cnames["CO"]=_CO;
    $cnames["CR"]=_CR;
    $cnames["CU"]=_CU;
    $cnames["CV"]=_CV;
    $cnames["CX"]=_CX;
    $cnames["CY"]=_CY;
    $cnames["CZ"]=_CZ;
    $cnames["DE"]=_DE;
    $cnames["DJ"]=_DJ;
    $cnames["DK"]=_DK;
    $cnames["DM"]=_DM;
    $cnames["DO"]=_DO;
    $cnames["DZ"]=_DZ;
    $cnames["EC"]=_EC;
    $cnames["EE"]=_EE;
    $cnames["EG"]=_EG;
    $cnames["EH"]=_EH;
    $cnames["ER"]=_ER;
    $cnames["ES"]=_ES;
    $cnames["ET"]=_ET;
    $cnames["EU"]=_EU;
    $cnames["FI"]=_FI;
    $cnames["FJ"]=_FJ;
    $cnames["FK"]=_FK;
    $cnames["FM"]=_FM;
    $cnames["FO"]=_FO;
    $cnames["FR"]=_FR;
    $cnames["FX"]=_FX;
    $cnames["GA"]=_GA;
    $cnames["GB"]=_GB;
    $cnames["GD"]=_GD;
    $cnames["GE"]=_GE;
    $cnames["GF"]=_GF;
    $cnames["GH"]=_GH;
    $cnames["GG"]=_GG;
    $cnames["GI"]=_GI;
    $cnames["GL"]=_GL;
    $cnames["GM"]=_GM;
    $cnames["GN"]=_GN;
    $cnames["GP"]=_GP;
    $cnames["GQ"]=_GQ;
    $cnames["GR"]=_GR;
    $cnames["GS"]=_GS;
    $cnames["GT"]=_GT;
    $cnames["GU"]=_GU;
    $cnames["GW"]=_GW;
    $cnames["GY"]=_GY;
    $cnames["HK"]=_HK;
    $cnames["HM"]=_HM;
    $cnames["HN"]=_HN;
    $cnames["HR"]=_HR;
    $cnames["HT"]=_HT;
    $cnames["HU"]=_HU;
    $cnames["ID"]=_ID;
    $cnames["IE"]=_IE;
    $cnames["IL"]=_IL;
    $cnames["IM"]=_IM;
    $cnames["IN"]=_IN;
    $cnames["IO"]=_IO;
    $cnames["IQ"]=_IQ;
    $cnames["IR"]=_IR;
    $cnames["IS"]=_IS;
    $cnames["IT"]=_IT;
    $cnames["JE"]=_JE;
    $cnames["JM"]=_JM;
    $cnames["JO"]=_JO;
    $cnames["JP"]=_JP;
    $cnames["KE"]=_KE;
    $cnames["KG"]=_KG;
    $cnames["KH"]=_KH;
    $cnames["KI"]=_KI;
    $cnames["KM"]=_KM;
    $cnames["KN"]=_KN;
    $cnames["KP"]=_KP;
    $cnames["KR"]=_KR;
    $cnames["KW"]=_KW;
    $cnames["KY"]=_KY;
    $cnames["KZ"]=_KZ;
    $cnames["LA"]=_LA;
    $cnames["LB"]=_LB;
    $cnames["LC"]=_LC;
    $cnames["LI"]=_LI;
    $cnames["LK"]=_LK;
    $cnames["LR"]=_LR;
    $cnames["LS"]=_LS;
    $cnames["LT"]=_LT;
    $cnames["LU"]=_LU;
    $cnames["LV"]=_LV;
    $cnames["LY"]=_LY;
    $cnames["MA"]=_MA;
    $cnames["MC"]=_MC;
    $cnames["MD"]=_MD;
    $cnames["ME"]=_ME;
    $cnames["MG"]=_MG;
    $cnames["MH"]=_MH;
    $cnames["MK"]=_MK;
    $cnames["ML"]=_ML;
    $cnames["MM"]=_MM;
    $cnames["MN"]=_MN;
    $cnames["MO"]=_MO;
    $cnames["MP"]=_MP;
    $cnames["MQ"]=_MQ;
    $cnames["MR"]=_MR;
    $cnames["MS"]=_MS;
    $cnames["MT"]=_MT;
    $cnames["MU"]=_MU;
    $cnames["MV"]=_MV;
    $cnames["MW"]=_MW;
    $cnames["MX"]=_MX;
    $cnames["MY"]=_MY;
    $cnames["MZ"]=_MZ;
    $cnames["NA"]=_NA;
    $cnames["NC"]=_NC;
    $cnames["NE"]=_NE;
    $cnames["NF"]=_NF;
    $cnames["NG"]=_NG;
    $cnames["NI"]=_NI;
    $cnames["NL"]=_NL;
    $cnames["NO"]=_NO;
    $cnames["NP"]=_NP;
    $cnames["NR"]=_NR;
    $cnames["NU"]=_NU;
    $cnames["NZ"]=_NZ;
    $cnames["OM"]=_OM;
    $cnames["PA"]=_PA;
    $cnames["PE"]=_PE;
    $cnames["PF"]=_PF;
    $cnames["PG"]=_PG;
    $cnames["PH"]=_PH;
    $cnames["PK"]=_PK;
    $cnames["PL"]=_PL;
    $cnames["PM"]=_PM;
    $cnames["PN"]=_PN;
    $cnames["PR"]=_PR;
    $cnames["PS"]=_PS;
    $cnames["PT"]=_PT;
    $cnames["PW"]=_PW;
    $cnames["PY"]=_PY;
    $cnames["QA"]=_QA;
    $cnames["RE"]=_RE;
    $cnames["RO"]=_RO;
    $cnames["RS"]=_RS;
    $cnames["RU"]=_RU;
    $cnames["RW"]=_RW;
    $cnames["SA"]=_SA;
    $cnames["SB"]=_SB;
    $cnames["SC"]=_SC;
    $cnames["SD"]=_SD;
    $cnames["SE"]=_SE;
    $cnames["SG"]=_SG;
    $cnames["SH"]=_SH;
    $cnames["SI"]=_SI;
    $cnames["SJ"]=_SJ;
    $cnames["SK"]=_SK;
    $cnames["SL"]=_SL;
    $cnames["SM"]=_SM;
    $cnames["SN"]=_SN;
    $cnames["SO"]=_SO;
    $cnames["SR"]=_SR;
    $cnames["ST"]=_ST;
    $cnames["SV"]=_SV;
    $cnames["SY"]=_SY;
    $cnames["SZ"]=_SZ;
    $cnames["TC"]=_TC;
    $cnames["TD"]=_TD;
    $cnames["TF"]=_TF;
    $cnames["TG"]=_TG;
    $cnames["TH"]=_TH;
    $cnames["TJ"]=_TJ;
    $cnames["TK"]=_TK;
    $cnames["TL"]=_TL;
    $cnames["TM"]=_TM;
    $cnames["TN"]=_TN;
    $cnames["TO"]=_TO;
    $cnames["TR"]=_TR;
    $cnames["TT"]=_TT;
    $cnames["TV"]=_TV;
    $cnames["TW"]=_TW;
    $cnames["TZ"]=_TZ;
    $cnames["UA"]=_UA;
    $cnames["UG"]=_UG;
    $cnames["UM"]=_UM;
    $cnames["US"]=_US;
    $cnames["UY"]=_UY;
    $cnames["UZ"]=_UZ;
    $cnames["VA"]=_VA;
    $cnames["VC"]=_VC;
    $cnames["VE"]=_VE;
    $cnames["VG"]=_VG;
    $cnames["VI"]=_VI;
    $cnames["VN"]=_VN;
    $cnames["VU"]=_VU;
    $cnames["WF"]=_WF;
    $cnames["WS"]=_WS;
    $cnames["YE"]=_YE;
    $cnames["YT"]=_YT;
    $cnames["ZA"]=_ZA;
    $cnames["ZM"]=_ZM;
    $cnames["ZW"]=_ZW;
    return $cnames;
}

function showVersion() {
    global $databasedriver,$vnumfull;
    if (function_exists('encodeSPQR')) { $extra=" for Germany"; } else { $extra=""; }  
    echo _YOU_ARE_USING_LOGAHOLIC.' '._LOGAHOLIC_PRODUCTNAME.' '._LOGAHOLIC_TRIAL.'version '.LOGAHOLIC_VERSION_NUMBER.$extra.' '._WITH_PHP_VERSION.' '.phpversion().' '._AND_DATABASE.' '.$databasedriver.' '._VERSION.' '.$vnumfull;
}

function printSegmentName() {
    global $trafficsource;
    if ($trafficsource > 0) {
        $source = getTrafficSourceByID($trafficsource);
        $segmentname = "<i>("._FILTERS.": <b>".$source["sourcename"]."</b>)</i>";
    } else {
        $segmentname="";   
    }
    return $segmentname;    
}

function UIDIalogButtons($show = array()) {
	$buttons = "<div class='dialog-controls'>";
	if(!empty($show)) {
		foreach($show as $button) {
			switch($button) {
				case "close":
					$buttons .= "<a title='Close this report' href='#' class='close'></a>";
					break;
				case "minimize":
					$buttons .= "<a title='Minimize This report' href='#' class='minimize'></a>";
					break;
				case "reload":
					$buttons .= "<a title='reload this report' href='#' class='reload'></a>";
					break;
				case "dialogsettings":
					$buttons .= "<a title='Change setting for this report' href='#' class='dialogsettings'></a>";
					break;
				default;
					break;
			}
		}
	} else {
		$buttons .= "<a title='Close this report' href='#' class='close'></a> <a title='Minimize This report' href='#' class='minimize'></a> <a title='reload this report' href='#' class='reload'></a> <a title='Change setting for this report' href='#' class='dialogsettings'></a>";
		$buttons .= "<a title='Expand this report' href='#' class='expand_switch expand'></a>";
	}
	$buttons .= "</div>";
	return $buttons;
}

function safeXML($xmlString = "") {
	$xmlString = iconv('ISO-8859-1', "UTF-8", $xmlString);
	$xmlString = str_replace("&", "&amp;", $xmlString);
	$xmlString = str_replace(">", "&gt;", $xmlString);
	$xmlString = str_replace("<", "&lt;", $xmlString);
	$xmlString = str_replace("'", "&apos;", $xmlString);
	$xmlString = str_replace('"', "&quot;", $xmlString);
	return $xmlString;
}

function CheckNotes($nlimit = 5, $noprint = false) {
	global $conf,$to,$from,$mysqlprefix,$db;

	if ($from) {
		$q=$db->Execute("select *,FROM_UNIXTIME(timestamp,'%d-%b-%Y %a') as day from ".TBL_NOTES." where profile='$conf' and (timestamp >=$from and timestamp <=$to or timestamp=\"0\") order by timestamp desc limit $nlimit");
		
		if($noprint == false) {
			$i=0;
			$disp="";
			while ($data=@$q->FetchRow()) {
				$disp.= "<font color=gray>".$data["day"]."</font> - ".$data["note"];
				$disp.= " &nbsp;&nbsp;<a class=graylink href=\"notes.php?conf=$conf&amp;donote=edit&amp;noteid=".$data["id"]."\">"._EDIT."</a>";
				$disp.= "  <a class=graylink href=\"notes.php?conf=$conf&amp;donote=del&amp;noteid=".$data["id"]."\">Del</a><br>";
				$i++;
			}
			$now=time();
			if (!@$disp) {
				echo "<div class=notes><table width=500 cellpadding=3 cellspacing=0 class=small border=0><tr class=small><td class=toplinegray valign=top>";
				echo "<a class=nodec3 href=\"notes.php?conf=$conf&amp;timestamp=$now&amp;donote=create\"><img src=\"images/icons/note_add.gif\" width=\"14\" height=\"14\" border=\"0\" align=\"left\" alt=\"\"></a> "._NO_NOTES.", <a class=nodec3 href=\"notes.php?conf=$conf&amp;timestamp=$now&amp;donote=create\">"._NEW_NOTE."</a></td></tr></table></div>";
			} else { 
				echo "<div class=notes><table width=500 cellpadding=3 cellspacing=0 class=small border=0><tr class=small bgcolor=\"#FEFEEA\"><td class=toplineyellow valign=top><img src=\"images/icons/notes.gif\" width=12 height=12 border=0 align=left alt=\"Notes\"></td><td class=toplineyellow valign=top>$disp</td><td valign=top class=toplineyellow>";
				echo "<a class=nodec3 href=\"notes.php?conf=$conf&amp;timestamp=$now&amp;donote=create\">"._MORE_NOTES."</a></td>";
				echo "</tr></table></div>";
			}
		} else {
			$notes = array();
			while ($data=@$q->FetchRow()) {
				$notes[] = $data;
			}
			
			return $notes;
		}
	}
}

function currentScriptURL($path=false) {
	if ($_SERVER['SERVER_PORT'] == '443') {
		$scriptURL = "https://";
	} else {
		$scriptURL = "http://";
	}
	if ($path==true) {
		return $scriptURL . $_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']);;
	} else {
		return $scriptURL . $_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'];
	}
}

function logaholic_url($update=false) {
	$url = getGlobalSetting("logaholic_url", false);
	if ($url == false || $update==true) {
		if (isset($_SERVER['HTTP_HOST'])) {
			$url = currentScriptURL(true);
			setGlobalSetting("logaholic_url",$url);			
		}
	}
	return $url;
}

function in_array_insensitive($needle, $haystack) {
	foreach($haystack as $value)
		{
			if (strcasecmp($value, $needle) == 0) {
				return true;
			}
		}
	return false;
}

function verifySettingsTable() {
	global $db,$databasedriver;
	$tablelist = $db->MetaTables();
  
  if (!in_array_insensitive(TBL_GLOBAL_SETTINGS, $tablelist)) {
	 // We need to create the user database.
	 $query = "CREATE TABLE ".TBL_GLOBAL_SETTINGS." ( ".
			"Name varchar(50) NOT NULL, ".
			"Profile varchar(50), ".
			"Value mediumtext, ".
			"PRIMARY KEY  (Name)".
			") ENGINE=MyISAM CHARSET=utf8";
	 $db->Execute($query) or die("Error creating GlobalSettings table, ". $db->ErrorMsg());
	 
     if ($databasedriver == "mysql") { $db->Execute("ALTER TABLE ".TBL_GLOBAL_SETTINGS." ADD INDEX ".TBL_GLOBAL_SETTINGS."_Profile (Profile, Name)"); 
     } else { $db->Execute("CREATE INDEX ".TBL_GLOBAL_SETTINGS."_Profile on ".TBL_GLOBAL_SETTINGS."(Profile, Name)"); }
     
	 // Insert the default record.
	} else {
		$columns = $db->MetaColumns(TBL_GLOBAL_SETTINGS);
		if (!isset($columns["PROFILE"])) {
			$query = "ALTER TABLE ".TBL_GLOBAL_SETTINGS." ADD COLUMN Profile varchar(50)";
			$db->Execute($query);
			if ($databasedriver == "mysql") { $db->Execute("ALTER TABLE ".TBL_GLOBAL_SETTINGS." ADD INDEX ".TBL_GLOBAL_SETTINGS."_Profile (Profile, Name)"); 
     } else { $db->Execute("CREATE INDEX ".TBL_GLOBAL_SETTINGS."_Profile on ".TBL_GLOBAL_SETTINGS."(Profile, Name)"); }
		}
	 // Could check to see if the structure is valid here, but we'll save that for a future need (when the structure changes)
	}
}

function getMultiProfileData($profile = 'Profiles', $settingname, $defaultvalue = "" ){
	global $db;
	
	if(!is_array($settingname)){
		return getProfileData($profile,$settingname,$defaultvalue);
	}
	
	$names = array();
	foreach($settingname as $setting){
		$names[] = " Name = \"" . $db->escape($setting['name']) . "\" ";
	}	
	$query = "SELECT Name,Value from ".TBL_GLOBAL_SETTINGS ." WHERE (".implode("OR",$names).") AND Profile = ".$db->Quote($profile);
	$result = $db->Execute($query);
	
	$data = array();
	while($row = $result->fetchRow()){
		$data[$row["Name"]] = $row["Value"];
	}
	
	# Check if there are settings missing. if so set them to their defaults
	foreach($settingname as $setting){	
		if(!isset($data[$setting["name"]])){
			$data[$setting["name"]] = $setting["default"];
		}
	}
	
	return $data;
}

function getProfileData($profile = 'Profiles', $settingname, $defaultvalue = "" ) {
	global $db;

	$query = "Select Value from ".TBL_GLOBAL_SETTINGS ." where Name = \"" . $db->escape($settingname) . "\" and Profile = ".$db->Quote($profile);
	if (!($result = @$db->GetRow($query))) {
		return $defaultvalue;
	}
	if (count($result) > 0) {
		return $result[0];
	} else {
		return $defaultvalue;
	}
}

function setProfileData($profile, $settingname, $settingvalue) {
	global $db;
	$query = "REPLACE into ".TBL_GLOBAL_SETTINGS." (Name, Value, Profile) values (" . $db->Quote($settingname) . ", " . $db->Quote($settingvalue) . ", " . $db->Quote($profile) .");";
	$db->Execute($query);
}

function deleteProfileData($profile, $settingname="") {
	global $db;
	$query = "DELETE from ".TBL_GLOBAL_SETTINGS." where Profile = ".$db->Quote($profile);
	if (($settingname) || ($settingname === NULL)) {
		if (strpos($settingname, "%")) {
			$query .= " and Name like ".$db->Quote($settingname);
		} else {
			$query .= " and Name = ".$db->Quote($settingname);
		}
	}
	$db->Execute($query);
}

function getGlobalSetting($settingname, $defaultvalue = "") {
	return getProfileData("", $settingname, $defaultvalue);
}

function setGlobalSetting($settingname, $settingvalue) {
	setProfileData("", $settingname, $settingvalue);
}

function logDebugMessage($error_message) {
	// This writes to a file (or does other things) if the debug flag is turned on...
	global $debug;
	if ($debug == 1) {
		// $_SESSION["debug_log"] .= $error_message . "<br>";
	}
}

function echoConsoleSafe($Message, $LogIfConsole = false) {
	global $running_from_command_line, $updatelog, $import;
	
	if(!empty($updatelog) && $LogIfConsole == true) {
		$import->LogProcess("#0|".strip_tags($Message).";");
	} else {
		if (!$running_from_command_line) {
			echo $Message;
		} else {
			if ($LogIfConsole) {
				$stripped = strip_tags($Message);  // Do we need to put a CR/LF on the end?  Maybe...
				echo $stripped;
			}
		}
	}
}
	
function getTrafficSources() {
	global $trafficsources, $supports_correlated_subselects, $db, $profile;
	
	# If we don't support correlated subselects, then we don't support traffic sources.
	if (!$supports_correlated_subselects) {
		return array();
	}
	
	if ($trafficsources) {
		return $trafficsources;
	}
	
	$trafficsources = array();
	
    $db->SetFetchMode(ADODB_FETCH_BOTH);
	$query = "SELECT * from ".TBL_TRAFFIC_SOURCES." where profileid='$profile->profileid' order by category, sourcename";
	if ($result = $db->Execute($query)) {
		while ($trafficsource = $result->FetchRow()) {
			$trafficsources[] = $trafficsource;
		}
	}
	return $trafficsources;
}

function getTrafficSourceByID($sourceID) {
    $sources = getTrafficSources();
	foreach ($sources as $source) {
		if ($source["id"] == $sourceID) {
			return $source;
		}
	}
	return null;
}

function printTrafficSourceSelect() {
	global $supports_correlated_subselects,$conf,$available,$session; 
    $sources = getTrafficSources();
	$selsource = @$_SESSION["trafficsource"];
	$result = "";
	$filter = "";
    	
	if (count($sources) > 0) {
		$result .= "<select class='report_option_field' id=\"trafficsource\" name=\"trafficsource\" ";
        if (@$available=="no") {
            $result .= "onChange=\"alert('"._FILTER_SYSTEM_NOT_ACTIVE_HERE."')\"";   
        }
        $result .=">\n";
		$result .= "  <option value=0>All Traffic / No Filter</option>\n";
        
		foreach ($sources as $source) {
			$result .= "  <option value=".$source["id"]. (($selsource == $source["id"]) ? " SELECTED" : "").">".$source["sourcename"]." (".$source["category"].")</option>\n";
			if ($selsource == $source["id"]) {
				$filter = $source["sourcecondition"];
			}
		}
		
		$result .=  "</select> &nbsp;&nbsp;";
		if($session->canEditProfiles()) {
			$result .= "<a class='optionlink open_iframe_window' href=\"filters.php?conf=$conf\">"._MANAGE_FILTERS."</a> ";
		}
		
	} else {
        if (!$supports_correlated_subselects) {
            $result.= _FILTERS_ONLY_SUPPORTED_BY_MYSQL_41;
        } else {
            $result .=_NO_FILTERS_CREATED_YET_PART1."<a class='optionlink' href=\"filters.php?conf=$conf\">"._NO_FILTERS_CREATED_YET_PART2."</a>";
        }
    }
    return $result;
}

function printTrafficSourceSelectUI($selsource = 0) {
	global $supports_correlated_subselects,$conf,$available,$session; 
    $sources = getTrafficSources();
	$result = "";
	$filter = "";
    	
	if (count($sources) > 0) {
		$result .= "<select class='isDefault' id='trafficsource' name='trafficsource' ";
        if (@$available=="no") {
            $result .= "onChange=\"alert('"._FILTER_SYSTEM_NOT_ACTIVE_HERE."')\"";   
        }
        $result .=">\n";
		$result .= "  <option value='0'". (($selsource == 0) ? " selected" : "").">All Traffic / No Filter</option>\n";
		foreach ($sources as $source) {
			$result .= "  <option value='{$source["id"]}'". (($selsource == $source["id"]) ? " selected" : "").">".$source["sourcename"]." (".$source["category"].")</option>\n";
			if ($selsource == $source["id"]) {
				$filter = $source["sourcecondition"];
			}
		}
		$result .= "</select>";
		if($session->canEditProfiles()) {
			$result .= "<a class='optionlink open_iframe_window' href=\"filters.php?conf=$conf\" style='color: #333;'>"._MANAGE_FILTERS."</a> ";
		}
	} else {
        if (!$supports_correlated_subselects) {
            $result .= _FILTERS_ONLY_SUPPORTED_BY_MYSQL_41;
        } else {
            $result .= _NO_FILTERS_CREATED_YET_PART1."<a class='optionlink open_iframe_window' href=\"filters.php?conf=$conf\">"._NO_FILTERS_CREATED_YET_PART2."</a>";
        }
    }
    return $result;
}

function prepareSegmentQuery($sqlstring) {
    global $profile;
    $joincond="";
    $addtables="";
     
    if (strpos("  ".$sqlstring," url ")!=FALSE) {
        $sqlstring=str_replace("url =","u.url=",$sqlstring);
        $sqlstring=str_replace("url !=","u.url!=",$sqlstring);
        $sqlstring=str_replace("url LIKE","u.url LIKE",$sqlstring);
        $sqlstring=str_replace("url NOT LIKE","u.url NOT LIKE",$sqlstring); 
        $addtables=",$profile->tablename_urls as u";
        $joincond="and a.url=u.id";       
    }
    
    
    if (strpos("  ".$sqlstring," params ")!=FALSE) {
        if (strpos("  ".$sqlstring," refparams ")!=FALSE) {
            # We do this so it doesn't get mixed up if both url params and ref params are used in the same query
            $sqlstring=str_replace("refparams","tempfix",$sqlstring);        
        }
        $sqlstring=str_replace("params ="," up.params=",$sqlstring);
        $sqlstring=str_replace("params !="," up.params!=",$sqlstring);
        $sqlstring=str_replace("params LIKE"," up.params LIKE",$sqlstring);
        $sqlstring=str_replace("params NOT LIKE"," up.params NOT LIKE",$sqlstring);
        $addtables.=",$profile->tablename_urlparams as up";
        $joincond.=" and a.params=up.id";
        if (strpos("  ".$sqlstring," tempfix ")!=FALSE) {
            # Ok, now reset it back to the original
            $sqlstring=str_replace("tempfix","refparams",$sqlstring);        
        }       
    }

    if (strpos("  ".$sqlstring," referrer ")!=FALSE) {
        $sqlstring=str_replace("referrer =","r.referrer=",$sqlstring);
        $sqlstring=str_replace("referrer !=","r.referrer!=",$sqlstring);
        $sqlstring=str_replace("referrer LIKE","r.referrer LIKE",$sqlstring);
        $sqlstring=str_replace("referrer NOT LIKE","r.referrer NOT LIKE",$sqlstring);
        $addtables.=",$profile->tablename_referrers as r";
        $joincond.=" and a.referrer=r.id";       
    }

    if (strpos("  ".$sqlstring," refparams ")!=FALSE) {
        $sqlstring=str_replace("refparams =","rp.params=",$sqlstring);
        $sqlstring=str_replace("refparams !=","rp.params!=",$sqlstring);
        $sqlstring=str_replace("refparams LIKE","rp.params LIKE",$sqlstring);
        $sqlstring=str_replace("refparams NOT LIKE","rp.params NOT LIKE",$sqlstring);
        $addtables.=",$profile->tablename_refparams as rp";
        $joincond.=" and a.refparams=rp.id";       
    }

    if (strpos("  ".$sqlstring," keywords ")!=FALSE) {
        $sqlstring=str_replace("keywords =","k.keywords=",$sqlstring);
        $sqlstring=str_replace("keywords !=","k.keywords!=",$sqlstring);
        $sqlstring=str_replace("keywords LIKE","k.keywords LIKE",$sqlstring);
        $sqlstring=str_replace("keywords NOT LIKE","k.keywords NOT LIKE",$sqlstring);
        $addtables.=",$profile->tablename_keywords as k";
        $joincond.=" and a.keywords=k.id";       
    }
    if (strpos("  ".$sqlstring," ipnumber ")!=FALSE) {
        $sqlstring=str_replace("ipnumber =","v.ipnumber=",$sqlstring);
        $sqlstring=str_replace("ipnumber !=","v.ipnumber!=",$sqlstring);
        $sqlstring=str_replace("ipnumber LIKE","v.ipnumber LIKE",$sqlstring);
        $sqlstring=str_replace("ipnumber NOT LIKE","v.ipnumber NOT LIKE",$sqlstring);
        $addtables.=",$profile->tablename_visitorids as v";
        $joincond.=" and a.visitorid=v.id";       
    }
   
    if (strpos("  ".$sqlstring," useragent ")!=FALSE) {
        $sqlstring=str_replace("useragent =","CONCAT(ua.name, ' ', ua.version)=",$sqlstring);
        $sqlstring=str_replace("useragent !=","CONCAT(ua.name, ' ', ua.version)!=",$sqlstring);
        $sqlstring=str_replace("useragent LIKE","CONCAT(ua.name, ' ', ua.version) LIKE",$sqlstring);
        $sqlstring=str_replace("useragent NOT LIKE","CONCAT(ua.name, ' ', ua.version) NOT LIKE",$sqlstring);
        # Avoid it being joined twice if we also have is_mobile in the sqlstring
        if (strpos("  ".$sqlstring," is_mobile ")===FALSE) { 
            $addtables.=",{$profile->tablename_useragents} as ua";
            $joincond.=" and a.useragentid=ua.id";
        }       
    }
    
    if (strpos($sqlstring,"is_mobile ")!==FALSE) {
        $sqlstring=str_replace("is_mobile =","ua.is_mobile=",$sqlstring);
        $sqlstring=str_replace("is_mobile !=","ua.is_mobile!=",$sqlstring);
        $sqlstring=str_replace("is_mobile LIKE","ua.is_mobile LIKE",$sqlstring);
        $sqlstring=str_replace("is_mobile NOT LIKE","ua.is_mobile NOT LIKE",$sqlstring);
        $addtables.=",{$profile->tablename_useragents} as ua";
        $joincond.=" and a.useragentid=ua.id";
    }
    
    $sqlinfo["joincond"]=$joincond;
    $sqlinfo["addtables"]=$addtables;
    $sqlinfo["sqlstring"]=$sqlstring;
	
	return $sqlinfo;
}

function makePositive($sqlstring) {
    $sqlstring=str_replace("!=","=",$sqlstring);
    $sqlstring=str_replace("NOT LIKE","LIKE",$sqlstring);
    return $sqlstring; 
}

function hasNegativeCondition($sqlstring) {
    if (strpos($sqlstring,"!=")!==FALSE || strpos($sqlstring,"NOT LIKE")!==FALSE) {
        return true;
    } else {
        return false;
    }        
    
}

function getOperator($sqlstring) {
    if (strpos($sqlstring," AND ")!==FALSE) {
        $operator="AND";
    } else {
        $operator="OR";
    }
    return $operator;       
}

function arrayConditions($sqlstring) {
    $operator=getOperator($sqlstring);
    $conditions = explode($operator,$sqlstring);
    return $conditions;
}

function getPositiveConditions($sqlstring) {
    $newstr="";
    $added=0;
    $operator=getOperator($sqlstring);
    $conditions = arrayConditions($sqlstring);
    foreach ($conditions as $value) {
        if ($added !=0) {
            $op=" $operator ";   
        } else {
            $op="";   
        }
        if (hasNegativeCondition($value)==false) {
             $newstr.=$op.$value;
             $added++;   
        }
    }
    if ($newstr!="") {
        return $newstr;   
    } else {
        return false;   
    }
    
}

function insertSegmentVisitorids($sqlinfo,$temp_tablename,$crawl="0") {
    global $db,$profile,$from,$to;
    $starttime=getmicrotime();
	
	$lt = "";
	if(!empty($sqlinfo['addtables'])){
		$lt = ",".str_replace(","," READ,",substr($sqlinfo['addtables'],1)) . "  READ";
	}
	
	$db->Execute("LOCK tables $temp_tablename WRITE, {$profile->tablename} as a READ $lt");
    $query = "INSERT INTO ".$temp_tablename." SELECT distinct a.visitorid traffic_source_visitor_visitorid from ".$profile->tablename.
                                    " as a {$sqlinfo['addtables']} where timestamp >= $from and timestamp <= $to and a.crawl = {$crawl} ".
                                    " {$sqlinfo['joincond']} and (".$sqlinfo['sqlstring'].")";                         
    $took = getmicrotime()-$starttime;
    $db->Execute($query); 
	$db->Execute("UNLOCK tables");	
}

function updateSegmentTable($sqlinfo,$temp_tablename) {
    global $db, $profile, $trafficsource, $from, $to; 
    
    # Now check if the from-to range is unchanged, if it is, we have to refill the table
    if (($from.$to)!=getProfileData($profile->profilename,$profile->profilename."SEGMENT_".$trafficsource."_Range",0)) {
        setProfileData($profile->profilename,$profile->profilename."SEGMENT_".$trafficsource."_Range",($from.$to));  
		//first clear whatever is in the table
        $db->Execute("delete from $temp_tablename"); 
		insertSegmentVisitorids($sqlinfo,$temp_tablename);
                  
    }    
    
}

function makeNewVisitorsSegmentTable($temp_tablename) {
    global $db,$profile,$from,$to;
    $db->Execute("delete from $temp_tablename");

	$db->Execute("LOCK tables $temp_tablename WRITE, {$profile->tablename_visitorids} as a READ");	
    $query = "INSERT INTO ".$temp_tablename." SELECT distinct a.id traffic_source_visitor_visitorid from ".$profile->tablename_visitorids.
                                    " as a where created >= $from and created <= $to";
                                                             
    $db->Execute($query);
	$db->Execute("UNLOCK tables");	
}

function makeReturnVisitorsSegmentTable($temp_tablename) {
    global $db,$profile,$from,$to;
    $days_ago=$from - (86400 * 180);
    $db->Execute("delete from $temp_tablename"); 
	
	$db->Execute("LOCK tables $temp_tablename WRITE, {$profile->tablename_visitorids} as a READ");
    $query = "INSERT INTO ".$temp_tablename." SELECT distinct a.id traffic_source_visitor_visitorid from ".$profile->tablename_visitorids.
                                    " as a where created < $from";
    $db->Execute($query);
	$db->Execute("UNLOCK tables");	
}

function subsetDataToSourceID($report_query, $trafficsource = null) {
	global $profile, $from, $to;
	global $db;
	if (!$trafficsource) {
		$trafficsource = @$_SESSION["trafficsource"];
	}
	if ($trafficsource) {
		
        $temp_tablename = "";
        $query = "";            
        # Make a semi permanent table ...
        $temp_tablename = $profile->tablename."_SEGMENT_".$trafficsource;  
        
        # Check if the table has been created
        if (!in_array_insensitive($temp_tablename, $db->MetaTables())) {        
            # Create the semi temporary table of visitors based on the traffic source.
            $query = "CREATE TABLE if not exists ".$temp_tablename." (traffic_source_visitor_visitorid int(11), KEY `visitorid` (`traffic_source_visitor_visitorid`)) ENGINE=MyISAM CHARSET=utf8";
            $db->Execute($query);
        }            
        # Get the primary query conditions
        $source = getTrafficSourceByID($trafficsource); 

        $sqlstring=$source["sourcecondition"];
        # We need to rewrite the sqlstring to the new style tables first 
        $sqlinfo = prepareSegmentQuery($sqlstring);        
        
        # We need to decode the source condition an then prepare it along these rules:
        # 1. We ALWAYS make a positive visitorid list, even for negative conditions
        # 2. If there is a positive condition, we always join on visitorid = visitorid in the end
        # 3. If there is only one condition and it is negative, we join on visitorid != visitorid in the end
        # 4. If there are positive AND negative conditions, we make T1 (positive list), T2 (negative list) and then T3 (all t1 that not in t2)                                                  

        $sqlstring = $sqlinfo["sqlstring"];
        
        $conditions = arrayConditions($sqlstring);
        $num_conditions = count($conditions);
        
        if ($source["sourcename"]=="Return Visitors") { # Return visitors
            $sqlinfo["matchmode"]="positive";
            makeReturnVisitorsSegmentTable($temp_tablename);        
            
        } else if ($source["sourcename"]=="New Visitors") { # New visitors
            $sqlinfo["matchmode"]="positive";
            makeNewVisitorsSegmentTable($temp_tablename);   

        } else if (hasNegativeCondition($sqlstring)==true && getPositiveConditions($sqlstring) == false) {
            $sqlinfo["matchmode"]="negative";
            $sqlinfo["sqlstring"] = makePositive($sqlstring);
            updateSegmentTable($sqlinfo,$temp_tablename);
               
        } else {
		
            $sqlinfo["matchmode"]="positive";
              
            if (hasNegativeCondition($sqlstring)==false) {
                # Just update the segment table
                updateSegmentTable($sqlinfo,$temp_tablename);
               
            } else {
                # This means we have more than one condition and at least one is negative
                if (($from.$to)!=getProfileData($profile->profilename,$profile->profilename."SEGMENT_".$trafficsource."_Range",0)) {
                    setProfileData($profile->profilename,$profile->profilename."SEGMENT_".$trafficsource."_Range",($from.$to));
                   
 				    # First we take the positive stuff and put it in t1
                    $db->Execute("CREATE TEMPORARY TABLE if not exists t1 (traffic_source_visitor_visitorid int(11), KEY `visitorid` (`traffic_source_visitor_visitorid`))");
                    $sqlinfo["sqlstring"] = getPositiveConditions($sqlstring);                
                    insertSegmentVisitorids($sqlinfo,"t1");
                    
                    # Then we take the negative ones and put them in t2
                    $db->Execute("CREATE TEMPORARY TABLE if not exists t2 (traffic_source_visitor_visitorid int(11), KEY `visitorid` (`traffic_source_visitor_visitorid`))");
                    $sqlinfo["sqlstring"] = makePositive($sqlstring);              
                    insertSegmentVisitorids($sqlinfo,"t2");
                                   
                    # Then we take everthing from t1 that is not found in t2 and we update the segment table manually
                    $t3 = "INSERT into $temp_tablename SELECT traffic_source_visitor_visitorid from t1 where traffic_source_visitor_visitorid NOT IN (select traffic_source_visitor_visitorid from t2)";
                    $db->Execute($t3);
                }                    
            } 
        }
        
        # OK, we have some data, now let's fix up the query that was passed in so it *joins* with the new table.        
        $aliasname = "FILTER_TABLE"; 
        if (strpos($report_query,"$profile->tablename as a")!=FALSE) {
            $m_alias="a.";   
        } else {
            $m_alias="";
        }
        
        # To prevent subqueries using IN from being editied, do this
        $rq = explode(" IN ", $report_query);
        if ($sqlinfo['matchmode']=="positive") {
            $report_query = preg_replace("/ from (.*) where (.*)/i", 
                                     " FROM ".$temp_tablename." as ".$aliasname.", $1 WHERE (".$aliasname.".traffic_source_visitor_visitorid = ".$m_alias."visitorid) and $2", 
                                     $rq[0]);
        } else {
            $report_query = preg_replace("/ where (.*)/i", 
                                     " WHERE ".$m_alias."visitorid NOT IN (select traffic_source_visitor_visitorid from ".$temp_tablename.") and $1", 
                                     $rq[0]);
         
        }
        # Stick the other IN clauses back on the query
        if (isset($rq[1])) {                             
            $report_query = $report_query ." IN ". $rq[1];
        }
        if (isset($rq[2])) {                             
            $report_query = $report_query ." IN ". $rq[2];
        }
        return $report_query;
    }
    return $report_query;
}
$trafficsources = null;

function LoadFieldSelect() {
  global $conf, $profile, $db;
  
  $output="<option></option>";
  $columns = $db->MetaColumns($profile->tablename);
  foreach ($columns as $column) {
    if ($column->name !="id" && $column->name !="timestamp" && $column->name !="status" && $column->name !="bytes" && $column->name !="crawl") {
      if ($column->name =="visitorid" ) {
         $output.="<option value=\"ipnumber\">ipnumber</option>\n";
      } else if ($column->name =="useragentid" ) {
         $output.="<option value=\"useragent\">useragent</option>\n";
      } else  {
        $output.="<option value=\"".$column->name."\">". $column->name . "</option>\n";
      }
    }
  }
  
  # Add one more for the mobile field
  $output.="<option value=\"is_mobile\">is Mobile</option>\n";
  
  return $output;
}

function LoadFieldSelectFull() {
  global $conf, $profile, $db;
  
  $output="<option></option>";
  
  $columns = $db->MetaColumns($profile->tablename);
  foreach ($columns as $column) {
    if ($column->name!="id" && $column->name!="timestamp" && $column->name!="sessionid") {
      if ($column->name =="visitorid" ) {
         $output.="<option value=\"ipnumber\">ipnumber</option>\n";
      } else if ($column->name =="useragentid" ) {
         $output.="<option value=\"useragent\">useragent</option>\n";
      } else  {
        $output.="<option value=\"".$column->name."\">". $column->name . "</option>\n";
      }
    }
	}
    
  # Add one more for the mobile field
  $output.="<option value=\"is_mobile\">is Mobile</option>\n";
  return $output;
}

function SelectField($html,$field) {
  # Remove any currently selected option
  $html=str_replace(" SELECTED", "",$html);
  
  # Now find the one we want and select it
  $field=trim($field);
  if ($field) {
    $html=str_replace("<option value=\"$field\">", "<option value=\"$field\" SELECTED>",$html);
  }
  return $html;
}

function theme_preview_image($str) {
	global $lo;
	eval(base64_decode($str));
}

function TimeOnPage($page) {
    global $db, $profile, $from, $to;
    /**
    * @desc This function will calculate the average time spent on a particular page, 
    * not counting people who bounced (had no further requests)
    * @returns average timespent on page in seconds
    */    

    # first find the visitorid and timestamp of the request we're interested in.
    $query="CREATE TEMPORARY TABLE goodtimes ";
    $query.="SELECT sessionid,min(timestamp) AS timestamp from $profile->tablename ";
    $query.="WHERE (timestamp >=$from and timestamp <=$to) ";
    $query.="AND url='".getID($page,"urls")."' AND crawl='0' ";
    $query.="GROUP BY sessionid";   
    
    # get the visitorid and timestamp of the request that came after the one we're interested in     
    $query2="INSERT INTO goodtimes ";
    $query2.="SELECT sessionid,min(timestamp) AS timestamp from $profile->tablename ";
    $query2.="WHERE (timestamp >=$from and timestamp <=$to) ";
    $query2.="AND referrer='".getID("http://$profile->confdomain".$page,"referrers")."' ";
    $query2.="AND crawl='0' ";
    $query2.="GROUP BY sessionid"; 
    
    # now create a tempoary table to hold the difference between the two timestamps
    $query3="CREATE TEMPORARY TABLE pagetime ";
    $query3.="SELECT (max(timestamp)-min(timestamp)) AS duration, sessionid ";
    $query3.="FROM goodtimes GROUP BY sessionid";
    
    # get rid of the ones that have no time
    $query4="DELETE from pagetime WHERE duration=0";
    
    # now calculate an average duration
    $query5="SELECT avg(duration) AS avgduration FROM pagetime";
    
    # issue the queries and get the data
    $db->Execute($query);
    $db->Execute($query2);
    $db->Execute($query3);
    $db->Execute($query4);
    $result = $db->Execute($query5); 
    $avg_duration = $result->FetchRow();
    
    # clean up temp tables (so we can do this more than once in a single script)
    $db->Execute("drop temporary table goodtimes");
    $db->Execute("drop temporary table pagetime");
    
    return $avg_duration['avgduration'];       
}

function SecurityCheck($input) {
	if(!is_array($input)) {
		if (strpos($input,">") !== FALSE) {
			echo "Security failure: $input (>): exiting (Invalid characters found)";
			exit();
		}
		if (strpos($input,"<") !== FALSE) {
			echo "Security failure: $input (<): exiting (Invalid characters found)";
			exit();
		}
		if (strpos($input,"{") !== FALSE) {
			echo "Security failure: $input ({): exiting (Invalid characters found)";
			exit();
		}
		if (strpos($input,"}") !== FALSE) {
			echo "Security failure (}): exiting (Invalid characters found)";
			exit();
		}
		if (strpos($input,"<script>") !== FALSE) {
			echo "Security failure : JAVASCRIPT INJECTION!"; 
			exit();
		}
		if (strpos($input,"</script>") !== FALSE) {
			echo "Security failure : JAVASCRIPT INJECTION!"; 
			exit();
		}
    }	
}

function get_month_lastday($month_num = FALSE, $year = FALSE) {
     # Set the proper month (current if non passed)
    $month_num = ($month_num) ? $month_num : date('m') ;
    $year = ($year) ? $year : date('Y') ;

    # return the last day of the requested month
    $d =  date('d', strtotime('-1 second',
            strtotime('+1 month', 
            strtotime($month_num.'/01/'.$year.' 00:00:00'))));
    return intval($d);
}

function DeleteRange($delprofile,$what) {
    global $db, $from, $to;
    /*
		what = 1: only delete detail data from main table
		what = 2: delete all
		what = 3: only delete from summary tables
    */
    if (($what==1) || ($what == 2)) {
        $db->Execute("delete from $delprofile->tablename where timestamp >=$from and timestamp <=$to");
		
        echo _DELETED_DETAIL_DATA_IN_DATE_RANGE."<br>";
    }
	
	if($what == 2) {
		DeleteDataFiles($delprofile,$from,$to);
	}
	
	$db->Execute("delete from $delprofile->tablename_conversions where timestamp >=$from and timestamp <=$to");
	$db->Execute("delete from $delprofile->tablename_screenres where timestamp >=$from and timestamp <=$to");
	$db->Execute("delete from $delprofile->tablename_colordepth where timestamp >=$from and timestamp <=$to");
	
	
    deleteProfileData($delprofile->profilename, "cache\_%");
	deleteProfileData($delprofile->profilename, $delprofile->tablename."%\_cachearray");
	deleteProfileData($delprofile->profilename, $delprofile->profilename.".cache\_%");
	deleteProfileData($delprofile->profilename, $delprofile->profilename."cache_trail");
	deleteProfileData($delprofile->profilename, $delprofile->profilename.".lastlogpos.%");
	deleteProfileData($delprofile->profilename, $delprofile->profilename.".firstlogline.%");
	deleteProfileData($delprofile->profilename, $delprofile->profilename.".lastmodtime.%");
}

function getField($table) {
    # Returns the appropriate field to use with table
    switch ($table) {
        case "urls":
            $field="url";
            break;
        case "urlparams":
            $field="params";
            break;
        case "keyword":
            $field="keywords";
            break;
        case "referrers":
            $field="referrer";
            break;
        case "refparams":
            $field="params";
            break;
        case "useragents":
            $field="useragent";
            break;
    }
    return $field;
}

function getID($item,$table,$profiletablename="") {
    /* returns the ID of the item in the table    
    */
    global $db, $profile; 
    
	if (empty($profiletablename)) {
		if (isset($profile->tablename)) {
			$profiletablename = $profile->tablename;
		} else {
			// this is no good
			return false;
		}		
	}
	
    $field = getField($table);    
    
    $query = "select id from ".$profiletablename."_$table where `hash`='".md5($item)."'";
    $result = $db->Execute($query);
    if ($data = $result->FetchRow()) {
		return $data['id'];
    } else {
        return false;
    }        
}

 # find or insert the item.
function InsertOrGetID($item,$table,$method="") {
    global $db, $profile, $timetracking;
   
    $starttime=getmicrotime();
    
    # get the field we need
    $field = getField($table);
    
	$item = substr($item,0,255);
	
	# Reject overly long 2 byte sequences, as well as characters above U+10000 and replace with ?
	$item = preg_replace('/[\x00-\x08\x10\x0B\x0C\x0E-\x19\x7F]'.
	 '|[\x00-\x7F][\x80-\xBF]+'.
	 '|([\xC0\xC1]|[\xF0-\xFF])[\x80-\xBF]*'.
	 '|[\xC2-\xDF]((?![\x80-\xBF])|[\x80-\xBF]{2,})'.
	 '|[\xE0-\xEF](([\x80-\xBF](?![\x80-\xBF]))|(?![\x80-\xBF]{2})|[\x80-\xBF]{3,})/S',
	 '?', $item );
	 
	# Reject overly long 3 byte sequences and UTF-16 surrogates and replace with ?
	$item = preg_replace('/\xE0[\x80-\x9F][\x80-\xBF]'.
	 '|\xED[\xA0-\xBF][\x80-\xBF]/S','?', $item );
	
    # Make the item safe for database insert
    $item = $db->quote($item);
    
    if ($method=="duplicate") {
        # This way is probably faster, but it's buggy on some mysql version AND we can't do this if we have merge tables
        $db->Execute("INSERT INTO ".$profile->tablename."_$table ($field, `hash`) VALUES ($item, MD5($item)) ON DUPLICATE KEY UPDATE id=LAST_INSERT_ID(id)"); 
        $id = $db->Insert_ID();
        $stoptime=getmicrotime();
        @$timetracking[$table]+=$stoptime-$starttime;
        return $id;            
    }
        
    $id = getID($item,$table);
    if ($id===false) {
        $db->Execute("Insert into ".$profile->tablename."_$table ($field, `hash`) values ($item, MD5($item))");
        $id = $db->Insert_ID();
    }
    $stoptime=getmicrotime();
    @$timetracking[$table]+=$stoptime-$starttime;  
    return $id;
}

function echoDebug($string, $stay = false,$call = true) {
	global $debug, $errorHandler;
	
	$extra_classes = "";
	if($stay == true) { $extra_classes .= " error"; }
	
	if(!empty($debug)) {
		ob_start();
		$errorHandler->showBacktrace();
		$callstack = ob_get_clean();
		echo "<div class=\"debug{$extra_classes}\">&nbsp; <div class='mysql_debug'>{$string}</div>";
		if($call === true){
			echo $callstack;
		}
		echo "</div>";
		// echo "<script type='text/javascript'>debugToConsole('');</script>";
	}
}
function echoWarning($string,$style="") {
    echo Warning($string,$style);
}
function Warning($string,$style="") {
    return "<div class=\"warning ui-state-error ui-corner-all\" style=\"$style\">$string</div>";
}
function echoNotice($string,$style="") {
    echo Notice($string,$style);
}
function Notice($string,$style="") {
    return "<div class=\"warning ui-state-highlight ui-corner-all\" style=\"$style\">$string</div>";
}
function Button($url,$anchor,$style="") {
    return "<a class=\"extrabuttons ui-state-default ui-corner-all\" style=\"$style\" href=\"$url\">$anchor</a>";   
}

/**
* @desc This function will take a table, create a copy, create an empty duplicate and 
* create a merge table that holds them both. It might also try to archive/zip 
* the copy of the original table
* @return Returns true on success or if it's already been done, false if something went wrong
*/
function ArchiveAndMergeTable($table) {
	global $db, $profile;
    
    # lets set up some table names
    $archivetable = $table."_archive";
    $currenttable = $table."_current";
    
    # we are only going to do this if it hasn't already been done, so lets check for that.
    $meta_tables = $db->MetaTables();
    if (in_array_insensitive($archivetable, $meta_tables)) {
        # nothing to do, return true cus it's done
        echoWarning("Table $archivetable already exists, so there is nothing to do.");
        return true;
    }
        
    # first lets get the create table query from the original table
    $query  = "SHOW CREATE TABLE $table";
    $q= $db->Execute($query);
    if ($data = $q->FetchRow()) {
        $createtable = $data['Create Table'];    
    } else {
        echoDebug("error for query: $query");
        return false;  
    }
    
    # now get the last id used so we can tell the new table to keep going from there
    if (strpos($createtable,"AUTO_INCREMENT")!==false) {
		$query  = "select id from $table order by id desc limit 1";
		$q= $db->Execute($query);
		if ($data = $q->FetchRow()) {
			$lastid=$data['id'];    
		} else {
			echoDebug("error for query: $query");
			return false;   
		}
		echo("the last id found is $lastid ... ");
	}
    
    # we're gonna rename the table now .... 
    $db->Execute("ALTER TABLE $table RENAME TO $archivetable");
    echoNotice("Renaming table $table to $archivetable .... done");
    
    # now we create a new, empty table
    $query = str_replace("`$table`","`$currenttable`",$createtable);
    $db->Execute($query);
    
    # if we have an auto increment field, we have to set the start value
    if (strpos($createtable,"AUTO_INCREMENT")!==false) {                                             
        $db->Execute("ALTER TABLE $currenttable AUTO_INCREMENT = $lastid");
    }
    echoNotice("Created a new table: $currenttable with structure:<br><pre>$query</pre>");
    
    # now lets create the merge table with the original tablename
    $query = $createtable;
    if (strpos($createtable,"PRIMARY KEY")!==false) {
        # we found a primary key and we can't have that in the merge table, so let's try to turn it into a normal index
        $query = str_replace("PRIMARY KEY","INDEX",$query);           
    }
    if (strpos($createtable,"UNIQUE KEY")!==false) {
        # no unique key allowed either
        $query = str_replace("UNIQUE KEY","INDEX",$query);           
    }
    $query = str_replace("ENGINE=MyISAM","ENGINE=MERGE UNION=($archivetable,$currenttable) INSERT_METHOD=LAST",$query);
    $db->Execute($query);
    echoNotice("Created new merge table: $table with structure:<br><pre>$query</pre>");
    echoWarning("Next, using a command line console (on *nix), you should pack the table like this:<br><b>myisampack $archivetable.MYI<br>myisamchk -rq --sort-index --analyze $archivetable.MYI</b><br>");
    return true;
}

/**
* @desc This will create an array with target files.
* If there are any entries with wildcards, we'll query 
* the database and add the variations
*/
function explodeTargets($targetfiles) {    
    global $profile,$db;
	
    $targets=explode(",",$targetfiles);
    $tarray = array();
    $i=0;
    $w=0;
    foreach ($targets as $thistarget) {
        $thistarget = trim($thistarget);
        if (strpos($thistarget,"*")!==FALSE) {
            # we have a wildcard, lets add all the variations later
            $wildcard[$w]=$thistarget;
            $w++;   
        } else {
            $tarray[$i] = $thistarget;
            $i++; 
        }           
    }
    if (isset($wildcard)) {
        foreach ($wildcard as $thistarget) {
            $query = "select url from $profile->tablename_urls where url like '".str_replace("*","%",$thistarget)."' limit 10";
            $q = $db->Execute($query);
            while ($data = $q->FetchRow()) {
                $tarray[$i] = $data['url'];
                $i++;
            }               
        }           
    }
    return $tarray;
}

function PrettyDate($time) {
    return date("D, d M Y / H:i",$time);    
}

function PrintLoadingBox($what="") {
    global $from, $to, $labels;
    if ($labels!="") {
        $what=$labels;   
    }
    $nicefrom = PrettyDate($from);
    $niceto = PrettyDate($to);
    ?>    
    <div id="loading" style="position:absolute;left:177px;top:149px;visibility:visible;z-index:11;">
    <TABLE bgcolor=white cellspacing=0 cellpadding=3 width="340">
    <TR><TD class=MoveableToplinegreen><FONT size="+1"><?php echo _BUILDING_REPORT?></FONT></TD></TR>
    <TR><TD bgcolor="#f0f0f0" class=dotline valign=middle><IMG src="images/Hourglass_icon.gif" width=32 height=32 alt="" border="0" align=left vspace=4 hspace=4>
    <?php echo _WAIT_WHILE_REPORT_IS_BEING_CREATED?><BR>
    <?php echo _NOW_CALCULATING?> <b><?php if(defined($what)) { echo constant($what); } else { echo $what; } echo "</b> "._DATE_FROM."<br>$nicefrom "._DATE_TO." $niceto"; ?>
    </TD></TR>
    </TABLE>
    </div>
    <script type="text/javascript">     
    $("#loading").draggable();
    </script>    
    <?php
    flush(); 
}

function deleteDataTable(&$profile) {
    global $databasedriver, $db; 
    @$db->Execute("drop table $profile->tablename");
    @$db->Execute("drop table $profile->tablename_conversions");
    @$db->Execute("drop table $profile->tablename_urls");
    @$db->Execute("drop table $profile->tablename_urlparams");
    @$db->Execute("drop table $profile->tablename_referrers");
    @$db->Execute("drop table $profile->tablename_refparams");
    @$db->Execute("drop table $profile->tablename_keywords");
    @$db->Execute("drop table $profile->tablename_visitorids");
    @$db->Execute("drop table $profile->tablename_screenres");
    @$db->Execute("drop table $profile->tablename_colordepth");
    @$db->Execute("drop table $profile->tablename_useragents");
	$tables = $db->Execute("show tables like '{$profile->tablename}\_%'");
    while ($table=$tables->FetchRow()) {
		$db->Execute("drop table {$table[0]}");
	}
}    

function createMainProfileTable($tablename) {
	global $databasedriver,$db;
	
	$tabcreate ="CREATE TABLE $tablename (".
			"timestamp int(11) NOT NULL default '0',    ".
			"visitorid int(11), ".
			"url int(11),    ".
			"params int(11),    ".
			"status smallint(3) NOT NULL default 0,    ".
			"bytes int(20) NOT NULL default 0,    ".
			"referrer int(11),    ".
			"refparams int(11),    ".
			"useragentid int(11),    ".
			"keywords int(11),    ".
			"country char(3) default '',    ".
			"crawl int(1) NOT NULL default '0', ".
			"sessionid int(11) ".
			") ENGINE=MyISAM CHARSET=utf8";
	$db->Execute($tabcreate);
	if ($databasedriver == "mysql") {
		$index_name_pre = "";
		$db->Execute("ALTER TABLE ".$tablename." ADD INDEX ".$index_name_pre."timestamp (timestamp)");
	} else {
		$index_name_pre = $tablename . "_";
		$db->Execute("CREATE INDEX ".$index_name_pre."timestamp ON ".$tablename."(timestamp)");
	}
}

/**
* Create the table structure for the profile table.
* @param SiteProfile The profile to update.
*/     
function createDataTable(&$profile) {
    global $databasedriver, $db;

  # Check and see if we need to create the table.
  $tablelist = $db->MetaTables();
  $tablename = $profile->tablename;
  
  
  if (!in_array_insensitive($tablename, $tablelist)) {
		createMainProfileTable($tablename);
		
    }
          
    # create summaries now in stead of in updatesummaries.php
    $meta_tables = $db->MetaTables();
    
    # create the tracker storage table 
    if ($profile->trackermode==1) {
       $db->Execute("CREATE TABLE if not exists $profile->tablename_trackerlog (".
            "id ". (($databasedriver != "sqlite") ? "int(11) NOT NULL auto_increment" : "integer primary key") . ", ".
            "logline TEXT default NULL".
            (($databasedriver != "sqlite") ? ", PRIMARY KEY (id)" : "").
        ") ENGINE=MyISAM CHARSET=utf8");
            
    }

   if (!in_array_insensitive($profile->tablename_conversions, $meta_tables)) {
        $db->Execute("CREATE TABLE $profile->tablename_conversions (
            timestamp int(11) NOT NULL default '0',
            visitorid varchar(32) NOT NULL default '0',
            url int(11) NOT NULL default '0'
        ) ENGINE=MyISAM CHARSET=utf8");
        if ($databasedriver == "mysql") {
            $db->Execute("ALTER TABLE {$profile->tablename_conversions} ADD INDEX {$profile->tablename_conversions}_timestamp (timestamp)");
        } else {
            $db->Execute("CREATE INDEX {$profile->tablename_conversions}_timestamp on {$profile->tablename_conversions}(timestamp)");
        }
        
    }
    
    if (!in_array_insensitive($profile->tablename_urls, $meta_tables)) {
        $db->Execute("CREATE TABLE $profile->tablename_urls (".
            "id int(11) NOT NULL auto_increment, ".
            "url varchar(255) NOT NULL default '0',".
            "title varchar(255) default '',".
			"`hash` CHAR(32) default '',".
            "PRIMARY KEY (id),".
            "UNIQUE KEY (`hash`)".
        ") ENGINE=MyISAM CHARSET=utf8");
    }
    if (!in_array_insensitive($profile->tablename_urlparams, $meta_tables)) {
        $db->Execute("CREATE TABLE $profile->tablename_urlparams (".
            "id int(11) NOT NULL auto_increment, ".
            "params varchar(255) default '', ".
			"`hash` CHAR(32) default '',".
            "PRIMARY KEY (id),".
			"UNIQUE KEY (`hash`)".
        ") ENGINE=MyISAM CHARSET=utf8");                                          
    }
    
    if (!in_array_insensitive($profile->tablename_referrers, $meta_tables)) {
        $db->Execute("CREATE TABLE $profile->tablename_referrers (".
            "id int(11) NOT NULL auto_increment, ".
            "referrer varchar(255) NOT NULL default '0',".
            "`hash` CHAR(32) default '',".
            "`internal_referrer` INT NOT NULL DEFAULT 0,".
            "PRIMARY KEY (id),".
			"UNIQUE KEY (`hash`)".
        ") ENGINE=MyISAM CHARSET=utf8");
    }
    if (!in_array_insensitive($profile->tablename_refparams, $meta_tables)) {
        $db->Execute("CREATE TABLE $profile->tablename_refparams (".
            "id int(11) NOT NULL auto_increment, ".
            "params varchar(255) default '',".
			"`hash` CHAR(32) default '',".
            "PRIMARY KEY (id),".
			"UNIQUE KEY (`hash`)".
        ") ENGINE=MyISAM CHARSET=utf8");
    }
    if (!in_array_insensitive($profile->tablename_keywords, $meta_tables)) {
        $db->Execute("CREATE TABLE $profile->tablename_keywords (".
            "id int(11) NOT NULL auto_increment, ".
            "keywords varchar(255) default '',".
			"`hash` CHAR(32) default '',".
            "PRIMARY KEY (id),".
			"UNIQUE KEY (`hash`)".
        ") ENGINE=MyISAM CHARSET=utf8");
    }

    if (!in_array_insensitive($profile->tablename_visitorids, $meta_tables)) {
        $db->Execute("CREATE TABLE $profile->tablename_visitorids (".
            "id int(11) NOT NULL auto_increment, ".
            "visitorid char(32) NOT NULL default '0',".
            "ipnumber varchar(255) NOT NULL default '0', ". 
            "authuser varchar(80), ".
            "created int(11) NOT NULL default '0', ".
            "customlabel varchar(80) NOT NULL default '',".
            "crawl int(1) NOT NULL default 0 ".
            ", PRIMARY KEY (id)".
            ", UNIQUE KEY (visitorid)".
        ") ENGINE=MyISAM CHARSET=utf8");
    }
    
	# sessionid table isn't used anymore
    if (!in_array_insensitive($profile->tablename_screenres, $meta_tables)) {
        $db->Execute("CREATE TABLE $profile->tablename_screenres (".
            "id ". (($databasedriver != "sqlite") ? "int(11) NOT NULL auto_increment" : "integer primary key") . ", ".
            "timestamp int(11) NOT NULL default '0',".
            "screenres varchar(20) NOT NULL default '',".
            "visits bigint(100) NOT NULL default '0'".
            (($databasedriver != "sqlite") ? ", PRIMARY KEY (id)" : "").
        ") ENGINE=MyISAM CHARSET=utf8");
        if ($databasedriver == "mysql") {
            $db->Execute("ALTER TABLE {$profile->tablename_screenres} ADD UNIQUE INDEX {$profile->tablename_screenres}_dayres (timestamp,screenres)");
        } else {
            $db->Execute("CREATE UNIQUE INDEX {$profile->tablename_screenres}_dayres on {$profile->tablename_screenres}(timestamp,screenres)");
        }
    }
    if (!in_array_insensitive($profile->tablename_colordepth, $meta_tables)) {
        $db->Execute("CREATE TABLE $profile->tablename_colordepth (".
            "id ". (($databasedriver != "sqlite") ? "int(11) NOT NULL auto_increment" : "integer primary key") . ", ".
            "timestamp int(11) NOT NULL default '0',".
            "colordepth varchar(20) NOT NULL default '',".
            "visits bigint(100) NOT NULL default '0'".
            (($databasedriver != "sqlite") ? ", PRIMARY KEY (id)" : "").
        ") ENGINE=MyISAM CHARSET=utf8");
        if ($databasedriver == "mysql") {
            $db->Execute("ALTER TABLE {$profile->tablename_colordepth} ADD UNIQUE INDEX {$profile->tablename_colordepth}_daycolor (timestamp,colordepth)");
        } else {
            $db->Execute("CREATE UNIQUE INDEX {$profile->tablename_colordepth}_daycolor on {$profile->tablename_colordepth}(timestamp,colordepth)");
        }
        
    }
    if (!in_array_insensitive($profile->tablename_useragents, $meta_tables)) {
		$db->Execute("CREATE TABLE IF NOT EXISTS `{$profile->tablename_useragents}` (
			  `id` int(11) NOT NULL AUTO_INCREMENT,
			  `name` varchar(255) DEFAULT NULL,
			  `version` varchar(255) DEFAULT NULL,
			  `os` varchar(255) DEFAULT NULL,
			  `os_version` varchar(255) DEFAULT NULL,
			  `engine` varchar(255) DEFAULT NULL,
			  `useragent` text,
			  `is_bot` int(1) NOT NULL default '0',
			  `is_mobile` int(1) NOT NULL default '0',
			  `device` varchar(255) DEFAULT NULL,
			  `hash` CHAR(32) NOT NULL default '0',
			  PRIMARY KEY (`id`),
			  UNIQUE KEY `hash` (`hash`)
			) ENGINE=MyISAM DEFAULT CHARSET=utf8;");
    }
	if (!in_array_insensitive($profile->tablename_crawl, $meta_tables)) {
		$db->Execute("CREATE TABLE IF NOT EXISTS `{$profile->tablename_crawl}` like {$profile->tablename}");
    }
    $profile->structure_version = CURRENT_PROFILE_STRUCTURE_VERSION;
	$profile->Save();	
    
}

function getAverage($data) {
	if(!empty($data)) {
		return (array_sum($data)/count($data));
	} else {
		return false;
	}
}
function getStandardDev($data) {
	if(!empty($data)) {
		$x_avg = getAverage($data);
		$n = count($data);
		foreach($data as $k=>$v) {
			$x_minus_avg[] = pow(($v - $x_avg),2);
		}
		$sigma = array_sum($x_minus_avg);
		$lastpart = ($sigma / ($n - 1));
		
		$return = sqrt($lastpart);
		return $return;
	} else {
		return false;
	}
}
function getCorrelation($data) {
	if(!empty($data)) {
		$i = 0;
		foreach($data as $k=>$v) {
			$days[] = $i++;
			$values[] = $v;
		}
		$n = count($data);
		if($n ==1){
			return false;
		}
		$day_avg = getAverage($days);
		$day_std = getStandardDev($days);
		$val_avg = getAverage($values);
		$val_std = getStandardDev($values);
		
		$x = 0;
		foreach($data as $k=>$v) {
			$x_part = ($days[$x] - $day_avg);
			$y_part = ($v - $val_avg);
			$e_part[] = ($x_part * $y_part); // stands for the mathematical E (sigma)
			$x++;
		}
		$e = array_sum($e_part); // mathematical E
		
		$lastpart = @($e / ($day_std * $val_std));
		$final = ($lastpart / ($n - 1));
		
		return $final;
	} else {
		return false;
	}
}
function getTrend($data,$keycol="") {
	$i = 0;
	$days = array();	
	$values = array();	
	foreach($data as $k => $v) {
		$days[] = $i;
		if ($keycol!="") {
			$values[] = $v[$keycol];
		} else {
			$values[] = $v;
		}
		
		$i++;
	}
	
	if($i == 1) { return false; }
	
	$day_std = getStandardDev($days);
	$val_std = getStandardDev($values);
	
	$r = getCorrelation($values);
	
	if ($day_std != 0) {
		$result = number_format((($val_std / $day_std) * $r),2,".","");
	} else {
		$result = 0;
	}
	
	return $result;
}    

function lgflush() {
    @ob_flush();
    flush();
    @ob_end_flush();
    ob_start();    
}

/**
* @desc This function takes the data array that is used to create report tables,
* grabs a single column from it and passes that back as a one-dimensional array  
*/
function ColumnArray($data,$col) {
    $a = array();
    foreach ($data as $row) {
        $a[] = $row[$col];
    }
    return $a;   
} 

/**
* @desc This function returns the first timestamp saved in datafiles.
* @returns timestamp
*/
function firstTimeDatafiles(){
	global $profile;
	# Get the correct year folder.
	$dir = $profile->datamanagerDir.$profile->profilename."/reports";
	$years = array();
	$list = glob($dir . '/*');
	if (!empty($list)) {
		foreach($list as $file) {
			$parts = explode("/",$file);
			$years[] = end($parts);
		}
	}
	sort($years);
	
	# get the first timstamp from visitors per day report.
	$dir = $dir."/".$years[0]."/_VISITORS_PER_DAY";
	$stamps = array();
	if(!file_exists($dir)){
		return 0;
	}	
	
	foreach(glob($dir . '/*') as $file) {
		$file = str_replace($dir,"",$file);
		$parts = explode(".",$file);
		if(strlen($parts[1]) > 6){
			$d = substr($parts[1],6,7);
			$m = substr($parts[1],4,-2);
			$y = substr($parts[1],0,4);
			$timestamp = mktime(0,0,0,$m,$d,$y);
			$stamps[] = $timestamp;
		}
	}
	sort($stamps);
	return (isset($stamps[0])) ? $stamps[0] : 0;
}

/**
* @desc This function returns the first timestamp ever recorded in the software.
* @returns timestamp
*/
function LogaholicFirstTimestamp(){
	global $profile;
	$from_db = GetMaxDateRange($profile);
	$from_db = $from_db["from"];
	$from_datafiles = firstTimeDatafiles();
	
	if($from_db < $from_datafiles || $from_datafiles == 0){
		return $from_db;
	} else {
		return $from_datafiles;
	}
	
}

/**
* @desc This function returns the maximum date range for a given profile
* i.e. the oldest date in the database as 'from' and the most recent date as 'to'
* @returns array containing from and to as keys
*/
function GetMaxDateRange($profile, $logtable=false) {
    global $db;
	if ($logtable==true) {
		$log = "log";
	} else {
		$log="";
	}
    $range = array();
    $q  = "select timestamp from {$profile->tablename}$log order by timestamp limit 1";
    $r = $db->Execute($q);
    $data=$r->FetchRow();
    $range['from'] = $data['timestamp'];
    
    $q  = "select timestamp from {$profile->tablename}$log order by timestamp desc limit 1";  
    $r = $db->Execute($q);
    $data=$r->FetchRow();
    $range['to'] = $data['timestamp'];
    
    return $range;           
}

/**
* @desc This function will enable or disable the ipencoding setting in the global.php file
*/
function toggleIPencoding() {
    global $ipencoding;
    $s = file_get_contents("files/global.php");
    # check if we already have the setting
    if (isset($ipencoding)) {
        if ($ipencoding==true) {
            $s = str_replace("ipencoding=true","ipencoding=false",$s);
            $msg = "IP encoding has been disabled! <br><br><b>Important:</b> If traffic was imported when IP encoding was enabled, those visitors will still have encoded IP numbers. We do not recommend switching IP encoding on and off in a live system. Just choose one settings and leave it that way.";
        } else {
            $s = str_replace("ipencoding=false","ipencoding=true",$s);
            $msg = "IP encoding has been enabled!";
        }
    } else {
        # if the setting was not present to begin with, we know we now have to enable it
        $s = str_replace("?>","\$ipencoding=true;\n?>",$s);
        $msg = "IP encoding has been enabled!";
    }
    if (file_put_contents("files/global.php",$s)===false) {
        return false;
    } else {
        return $msg;    
    }        
}

/**
* @desc This functions takes a path and makes sure we only use forward slashes and adds a slash at the end if it's not there
*/
function properSlash($path) {
    $path = str_replace("\\","/",$path);
    if (substr($path,-1)!="/") {
        $path.="/";
    }
    return $path;    
}

/**
* @desc This function sorts any multidimensional array on the selected row 
*/
function DataSort($data,$sort_row,$order=SORT_DESC) {
    foreach ($data as $key => $row) {
        $sortkey[$key] = $row[$sort_row];
    }
	if (is_array($sortkey)) {
		array_multisort($sortkey, $order, $data);
	}
    return $data;
}

/**
* @desc Check if any maintenance is needed
*/
function housekeeping() {
	global $profile, $loginSystemExists;
	
	// Does the current profile need to be updated?	
	if (isset($profile)) {
		if ($profile->profileloaded) {
			// make sure the profile is of adequate version to be used.
			if ($profile->structure_version < CURRENT_PROFILE_STRUCTURE_VERSION) {
				include_once "version_check.php";
				updateDataTableForProfile($profile);
				$profile->Load($profile->profilename); // Load the profile again to apply any changes
			}
		}
	}
}

/**
* @desc SORT OUT THE REPORT CATEGORYS AND THE REPORTS INSIDE THE CATEGORY
*/
function sort_reports($reports) {
	# first of all, we want to set an order for all known report categories
	$cat_order[] = "_VISITOR_DETAILS";
	$cat_order[] = "_POPULAR_CONTENT";
	$cat_order[] = "_INCOMING_TRAFFIC";
	$cat_order[] = "_PERFORMANCE";	
	$cat_order[] = "_CLIENT_SYSTEM";
	$cat_order[] = "_TRAFFIC";
	$cat_order[] = "_SOCIAL_MEDIA";
	$cat_order[] = "_FACEBOOK_FRIENDS";
	$cat_order[] = "_FACEBOOK_APPS_PAGES";

	# next, split the reports array into seperate arrays, one for each category
	$cat_reports = array();
	foreach($reports as $key => $val) {
		$cat_reports[$val["Category"]][$key] = $val;
		// in case there are any categories we don't know about, make sure to add them to the end of the cat_order array
		if (!in_array($val["Category"],$cat_order)) { $cat_order[] = $val["Category"]; }
	}
	
	# now, sort the reports inside each category by the order element
	foreach($cat_reports as $cat => $report_array) {
		# since we can't be sure each report actually has an Order element, we have to verify them
		$report_array = verifyOrderReports($report_array);
		$new_reports[$cat] = DataSort($report_array, "Order", SORT_ASC);
	}
	
	# merge it all back together using the right category order
	$reports = array();
	foreach($cat_order as $cat) {
		if(!empty($new_reports[$cat])) {
			$reports = array_merge($reports,$new_reports[$cat]);
		}
	}
	
	return $reports;
}

function verifyOrderReports($reports) {	
	# find the max value
	$max = multimax($reports,"Order");
	# now fill her up
	$verified = array();
	foreach($reports as $key => $value) {
		if (!isset($value["Order"])) { $value["Order"] = $max++; }
		$verified[$key] = $value;
	}
	return $verified;
}

/**
* @desc this function returns the max value from a multidemensional array, for the given key
*/
function multimax($array, $key) {
	$max = 0;
	foreach($array as $val) {
		if (isset($val[$key]) > $max) { $max = $val[$key]; }
	}
	return $max;
}

function dump($var) {
	echo "<pre>";
	print_r($var);
	echo "</pre>";
}

function stdToAssoc($obj) {
	$array = array();
	
	foreach($obj as $key => $value) {
		if(is_object($value) == true || is_array($value) == true) {
			$array[$key] = stdToAssoc($value);
		} else {
			$array[$key] = $value;
		}
	}
	
	return $array;
}

function getReportArray() {
	$path = logaholic_dir() . "reports/";
	$report_dir = opendir($path);
	while ($file = readdir($report_dir)) {
		if (strpos($file,".php")!==false) {
			$str = file_get_contents($path . $file);
			$str = substr($str,strpos($str,"\$reports["));
			$str = substr($str,0,(strpos($str,");")+2));
			eval($str);
		}	
	}
	closedir($report_dir);
	return $reports;
}

function SetupCustomDateFormat($timeformat){
	if(empty($timeformat)) {
		$timeformat = array(
			"format1" => "m",
			"seperator1" => "-",
			"format2" => "d",
			"seperator2" => "-",
			"format3" => "Y",
			"seperator3" => " ",
			"format4" => ""				
		);
	}
	?>			
	<select class="format1" name="format1">
		<option value="d" <?php if($timeformat["format1"] == "d"){ echo 'selected="selected"'; } ?>>Day (31)</option>
		<option value="D" <?php if($timeformat["format1"] == "D"){ echo 'selected="selected"'; } ?>>Day (Mon)</option>
		<option value="m" <?php if($timeformat["format1"] == "m"){ echo 'selected="selected"'; } ?>>Month (12)</option>
		<option value="M" <?php if($timeformat["format1"] == "M"){ echo 'selected="selected"'; } ?>>Month (Dec)</option>
		<option value="Y" <?php if($timeformat["format1"] == "Y"){ echo 'selected="selected"'; } ?>>Year</option>
		<option value="" <?php if($timeformat["format1"] == ""){ echo 'selected="selected"'; } ?>>None</option>
	</select>
	<select class="seperator1" name="seperator1">
		<option value=" " <?php if($timeformat["seperator1"] == " "){ echo 'selected="selected"'; } ?>> </option>
		<option value="-" <?php if($timeformat["seperator1"] == "-"){ echo 'selected="selected"'; } ?>>-</option>
		<option value="/" <?php if($timeformat["seperator1"] == "/"){ echo 'selected="selected"'; } ?>>/</option>
		<option value="," <?php if($timeformat["seperator1"] == ","){ echo 'selected="selected"'; } ?>>, </option>
	</select>
	<select class="format2" name="format2">
		<option value="d" <?php if($timeformat["format2"] == "d"){ echo 'selected="selected"'; } ?>>Day (31)</option>
		<option value="D" <?php if($timeformat["format2"] == "D"){ echo 'selected="selected"'; } ?>>Day (Mon)</option>
		<option value="m" <?php if($timeformat["format2"] == "m"){ echo 'selected="selected"'; } ?>>Month (12)</option>
		<option value="M" <?php if($timeformat["format2"] == "M"){ echo 'selected="selected"'; } ?>>Month (Dec)</option>
		<option value="Y" <?php if($timeformat["format2"] == "Y"){ echo 'selected="selected"'; } ?>>Year</option>
		<option value="" <?php if($timeformat["format2"] == ""){ echo 'selected="selected"'; } ?>>None</option>
	</select>
	<select class="seperator2" name="seperator2">
		<option value=" " <?php if($timeformat["seperator2"] == " "){ echo 'selected="selected"'; } ?>> </option>
		<option value="-" <?php if($timeformat["seperator2"] == "-"){ echo 'selected="selected"'; } ?>>-</option>
		<option value="/" <?php if($timeformat["seperator2"] == "/"){ echo 'selected="selected"'; } ?>>/</option>
		<option value="," <?php if($timeformat["seperator2"] == ","){ echo 'selected="selected"'; } ?>>, </option>
	</select>
	<select class="format3" name="format3">
		<option value="d" <?php if($timeformat["format3"] == "d"){ echo 'selected="selected"'; } ?>>Day (31)</option>
		<option value="D" <?php if($timeformat["format3"] == "D"){ echo 'selected="selected"'; } ?>>Day (Mon)</option>
		<option value="m" <?php if($timeformat["format3"] == "m"){ echo 'selected="selected"'; } ?>>Month (12)</option>
		<option value="M" <?php if($timeformat["format3"] == "M"){ echo 'selected="selected"'; } ?>>Month (Dec)</option>
		<option value="Y" <?php if($timeformat["format3"] == "Y"){ echo 'selected="selected"'; } ?>>Year</option>
		<option value="" <?php if($timeformat["format3"] == ""){ echo 'selected="selected"'; } ?>>None</option>
	</select>
	<select class="seperator3" name="seperator3">
		<option value=" " <?php if($timeformat["seperator3"] == " "){ echo 'selected="selected"'; } ?>> </option>
		<option value="-" <?php if($timeformat["seperator3"] == "-"){ echo 'selected="selected"'; } ?>>-</option>
		<option value="/" <?php if($timeformat["seperator3"] == "/"){ echo 'selected="selected"'; } ?>>/</option>
		<option value="," <?php if($timeformat["seperator3"] == ","){ echo 'selected="selected"'; } ?>>, </option>
	</select>
	<select class="format4" name="format4">
		<option value="d" <?php if($timeformat["format4"] == "d"){ echo 'selected="selected"'; } ?>>Day (31)</option>
		<option value="D" <?php if($timeformat["format4"] == "D"){ echo 'selected="selected"'; } ?>>Day (Mon)</option>
		<option value="m" <?php if($timeformat["format4"] == "m"){ echo 'selected="selected"'; } ?>>Month (12)</option>
		<option value="M" <?php if($timeformat["format4"] == "M"){ echo 'selected="selected"'; } ?>>Month (Dec)</option>
		<option value="Y" <?php if($timeformat["format4"] == "Y"){ echo 'selected="selected"'; } ?>>Year</option>
		<option value="" <?php if($timeformat["format4"] == ""){ echo 'selected="selected"'; } ?>>None</option>
	</select>
	<?php
}
$phpdatepicker_date_format = array(
"d" => "dd"
,"D" => "D"
,"m" => "mm"
,"M" => "M"
,"Y" => "yy"
);
function GetCustomDateFormat($script = "PHP", $returnasarray = FALSE){
	global $dateFormat,$profile,$phpdatepicker_date_format;
	# returns a date string
	if(!isset($profile->profilename)) {
		$date = getGlobalSetting("profileDateFormat",$dateFormat);
	} else {
		$date = getProfileData($profile->profilename, $profile->profilename.".profileDateFormat", getGlobalSetting("profileDateFormat",$dateFormat));
	}
	$date = unserialize($date);
	if($returnasarray == TRUE){
		return $date;
	}
	$ds = "";
	foreach($date as $k => $v){
		if($script == "PHP"){
			$ds .= $v;
		}
		if($script == "JS"){
			if(array_key_exists($v,$phpdatepicker_date_format)){
				$ds .= $phpdatepicker_date_format[$v];
			} else {
				$ds .= $v;
			}
		}
	}
	return trim($ds);
}


# recursively remove a directory
function rrmdir($dir) {
    if (!file_exists($dir)) {
		return;
	}
	$list = glob($dir . '/*');
	if (!empty($list)) {
		foreach($list as $file) {
			if(is_dir($file)){
				rrmdir($file);
			}else{
			   unlink($file);
			}
		}
	}
	if(is_dir($dir)){
		rmdir($dir);
	}
}

# Set chmod
function set_permissions($file){
	if (@chmod($file, 0777) == false) {
		return false;
	}
	if(function_exists('posix_geteuid')){
		clearstatcache();
		$u = posix_geteuid();
		if($u == fileowner($file) || $u==0) {
			return true;
		}
	}
	return true;
}

function DeleteDataFiles($profile,$from = '',$to = ''){
	global $reports;
	
	$dir = "{$profile->datamanagerDir}{$profile->profilename}/reports";
	if(empty($from) && empty($to)) {
		rrmdir($dir);
		echo "Succesfully deleted all the datafiles.";
		return;
	}
	
	foreach(glob($dir . '/*') as $yearFolders) {
		foreach(glob($yearFolders . '/*') as $reportFolder) {
			if (count(glob($reportFolder . '/*')) < 2) {
				# dir is empty, just continue
				continue;
			}
			foreach(glob($reportFolder . '/*') as $datafile) {	
				$full_path = $datafile;
				$datafile = explode($reportFolder."/",$datafile);
				$datafile = $datafile[1];
				
				$parts = explode(".",$datafile);
				# before getting the date, we have to compensate for any dites in the profilename
				$c = substr_count($profile->profilename, ".");
				$date = $parts[1+$c];
				#check if this is a day or a month file.
				if(strlen($date) == 8) {
					#this is a day file.
					$stamp = mktime(0,0,0,substr($date,4,-2),substr($date,6,7),substr($date,0,4));
				} else if (strlen($date) == 6) {
					#this is a month file.
					$stamp = mktime(0,0,0,substr($date,-2),01,substr($date,0,4));
				}
				if(isset($stamp)) {
					if($stamp >= $from && $stamp <= $to){
						unlink($full_path);
					}					
				}
			}
		}
	}
	echo "Succesfully deleted the datafiles.";
}

function Post($url,$data) {	
		
		# Create a stream
		$opts = array(
		  'http' => array(
			'method' => "POST",
			'header' => 'Content-Type: application/x-www-form-urlencoded',
			'content' => http_build_query($data)
		  )
		);

		$context = stream_context_create($opts);		
		# Open the file using the HTTP headers set above
		$file = file_get_contents($url, false, $context);
		return $file;
}

function CheckIfIndexExists($table,$index){
	global $db;
	$check = $db->Execute("SHOW CREATE TABLE {$table}");
	$row = $check->fetchRow();
	$statement = $row["Create Table"];
	$index = "KEY `".$index;
	if(strpos($statement,$index) === false){
		return false;
	} else {
		return true;
	}
}
?>