<?php
set_time_limit(86400);

# These are the files we need to include.
include_once("common.inc.php");

include_once("logparser.inc.php");
include_once("includes/datamanager.php");
include_once("components/import/import.php");
ob_start();

# Startup the class
$import = new Import();
$import->manage_keys=true;

# Do the availability checks; if one of them fails, the script will die and return the error message.
$canDoPerlUpdate = getGlobalSetting("canDoPerlUpdate", 'N/A');
if($canDoPerlUpdate == 'N/A') {
	$perlCheck = $import->AvailableCheck();
	if($perlCheck['hasError'] !== false) {
		foreach($perlCheck as $perlError) {
			echo $perlError."<br/>";
		}
		
		setGlobalSetting("canDoPerlUpdate", 0);
		lgflush();
		// should we throw them back to update-php here ?
		include "update-php.php";
		exit;
	} else {
		setGlobalSetting("canDoPerlUpdate", 1);
	}
}

# We are running a perl update, so we set a flag.
setProfileData($profile->profilename, "{$profile->profilename}.update_running", 'yes');

# Open the Update Progress Log, but first check if the dir exists.
if (!file_exists($profile->datamanagerDir.$profile->profilename)) {	
	mkdir($profile->datamanagerDir.$profile->profilename); 
	set_permissions($profile->datamanagerDir.$profile->profilename);
}
$progress_log = $profile->datamanagerDir.$profile->profilename."/{$profile->profilename}_update_progress.lwa.log";
$updatelog = fopen($progress_log, "w");
set_permissions($progress_log);


if (!$running_from_command_line) {
	echoNotice("Your update is running in the background. You don't have to keep this page open.<br/>You can <a href='javascript:showUpdateProgress()'>view update progress</a> here.<br/>Please note that you might experience Logaholic being slower than you're used to, while updating.", 'margin: 5px 10px;');
} else {
	echo "\nProgress Log = ".trim($progress_log). "\n";
	echoConsoleSafe("Your update is running in perl mode...\n ",true);
}

lgflush();


# Check if we have move to done function activated.
if($movedone === true && !file_exists($profile->logfilefullpath.'done/')){
	echoWarning("We are unable to find the '<b>done</b>' directory at the following location:<br/><ul><li>{$profile->logfilefullpath}</li></ul>
	Please create a '<b>done</b>'  directory in your logfile directory <b>({$profile->logfilefullpath})</b><br/>
	We need this to place the completed logfiles in <b>{$profile->logfilefullpath}done/</b>","margin:10px;");
	setProfileData($profile->profilename,"{$profile->profilename}.update_running","no");
	exit();
}

# We want to be able to update while the user is somewhere else, or is gone.
ignore_user_abort(true);

# We want to know how long an update takes.
$st_time = time();

# Get the log files in array format
$files = $import->GetLogFilesArray();

# lets get started
$import->LogProcess("We're starting an update job. System time is now ".date("Y-m-d H:i:s",time()), false);
$import->LogProcess("Using Perl parser", false);

$import->correctProfileTablename();

$import->EnableKeys($profile->tablename, false);

$skiptime = $import->getStartTimeForProfile();

# first, check if we need to run datamanager
$dataManager = new DataManager();
$dataManager->print = 2;
$dataManager->AutoManager($profile);

# For each log file, we'll parse it, and insert it into the database, among other stuff; keep reading to discover more.
foreach($files as $file) {
	if (!file_exists($file)) {
		$import->LogProcess("$file does not exist");
		continue;
	}
	
	if (filesize($file)==0) {
		$import->LogProcess("$file is empty");
		//echo ' skip';
		continue;
	} 
	
	# Create a empty array for the bandwidth
	$bandwidth = array();
	
	# first, check if we need to run datamanager
	$dataManager = new DataManager();
	$dataManager->print = 2;
	$dataManager->AutoManager($profile);
	
	$load = sys_getloadavg();
	$import->LogProcess("System load {$load[0]} {$load[1]} {$load[2]}");
	$start_file_processing = time();
	
	# We set a profile setting so we can keep track how long update is running for the current file.
	setProfileData($profile->profilename, $profile->profilename.".perlupdate_modtime", time());
	
	# This variable is being set to let the user know which file we're currently updating.
	$human_readable_file = explode("/", $file);
	$human_readable_file = $human_readable_file[(count($human_readable_file) - 1)];
	
	$size = round((filesize($file) / 1024) / 1024, 2);
	$format = formatOfLogFile($file);
	
	$perlformats = array('ApacheCombinedLogParser','LooseApacheCommonLogParser','ApacheCombinedCookieLogParser');
	if(!in_array($format['ClassName'],$perlformats)){
		$php_fallback = true;
		include_once "update-php.php";
		exit;
	}
	
	$import->LogProcess(_PARSING_LOG_FILE." ({$size} mb, {$format['ClassName']}) ...", false);
	
	if ($import->setLogParser($format['ClassName'])!==false) {
		if (substr($profile->logfilefullpath.$file, -3, 3) == ".gz") {
			
			$filehandle = new gzhandle();
		} else {
			$filehandle = new fhandle();
		}		
		$logfile = $filehandle->fileopen($file, 'r');
		$import->log_parser->Initialize($logfile);
		$filehandle->fileclose($logfile);
	} else {
		$import->LogProcess("Unknown log file format");
		continue;
	}	
	
	# Set the configuration file location
	$config_file = logaholic_dir()."files/{$profile->profilename}_config";
	
	# Set the perl script command
	if ($profile->profilename=="oldperl") {
		$perl_script = "perl ".logaholic_dir()."components/import/preparse.plx {$config_file}";
	} else {
		$perl_script = "perl ".logaholic_dir()."components/import/preparse.pl -profile_config {$config_file} -schema_version v1";
	}
	$import->LogProcess($perl_script);
	
	# Set the preparsed log file location (The file where Perl writes to).
	$new_log_file = "{$mysqltmp}{$profile->profilename}_lwa_log";
	
	# Create the configuration file.
	$import->WriteConfigFile($config_file, $file, $new_log_file);	
	
    # Execute the Perl script
    $descriptorspec = array(
            // no input pipe to child
            1 => array("pipe", "w"),  // stdout pipe that child will write to
            2 => array("pipe", "w")   // stderr pipe that child will write to
    );
	//exit();
    $cwd = logaholic_dir()."components/import/";
    $process = proc_open($perl_script, $descriptorspec, $pipes, $cwd);
    $p_error;
    $p_output;
    $return_value;
    if (is_resource($process)) {
        $p_output = stream_get_contents($pipes[1]);
        fclose($pipes[1]);
        $p_error = stream_get_contents($pipes[2]);
        fclose($pipes[2]);
        $return_value = proc_close($process);
    }
    $p_error = ($p_error) ? $p_error : '';
    $return_value = ($return_value) ? $return_value : 0;
    $import->LogProcess("Perl process exit value:" . $return_value);
    $import->LogProcess("Perl process STDERR:" . $p_error);

	$output = explode("\n", $p_output);
	$took = $import->thisTook($start_file_processing);	

	$import->LogProcess("--- Start Perl Output ---");
	# For each line of output coming from our perl script, we'll parse it.
	// dump($output);
	foreach($output as $val) {
		$import->LogProcess($val);
		# If the line defines the last known log position, set it in a PHP variable.
		if(strpos($val, 'lastlogpos:') !== false) {
			$v = explode(':', $val);
			$lastlogpos = $v[1];
		}
		
		# If the line defines the first log line, set it in a PHP variable.
		if(strpos($val, 'firstlogline:') !== false) {
			$v = explode(':', $val);
			$firstlogline = md5($v[1]);
		}
		
		# If the line defines the sessioncounter, set it in a PHP variable.
		if(strpos($val, 'sessioncounter:') !== false) {
			$v = explode(':', $val);
			$sessioncounter = $v[1];
		}
		
		# Merge the bandwidth data coming from Perl with the existing data.
		if(strpos($val, 'bandwidth:') !== false) {
			$v = explode(':', $val);
			$v = explode('=', $v[1]);
			
			# If we already have bandwidth data for the current date/time, add it.
			if(isset($bandwidth[$v[0]])) {
				$bandwidth[$v[0]] = $bandwidth[$v[0]] + $v[1];
			} else {
				# Else we want to set it for this date/time.
				$bandwidth[$v[0]] = $v[1];
			}
		}
	}
	$import->LogProcess("--- End Perl Output ---");
	unset($output);

	# If there is no last log position, set it to zero.
	if(!isset($lastlogpos)) {
		$lastlogpos = 0;
	}
	
	# If there is no first log line, make it empty.
	if(!isset($firstlogline)) {
		$firstlogline = '';
	}
	
	
	# store the bandwidth in datafiles
	$import->StoreBandwidth($bandwidth);
	
	# Save the bandwidth data, first log line and last log position.
	setProfileData($profile->profilename, $profile->profilename.".lastlogpos.".md5($file), $lastlogpos);
	setProfileData($profile->profilename, $profile->profilename.".firstlogline.".md5($file), md5($firstlogline));
	setProfileData($profile->profilename, $profile->profilename.".lastmodtime.".md5($file), filemtime($file));
	setProfileData($profile->profilename, $profile->profilename.".sessioncounter", $sessioncounter);
	
	# Import the data parsed by perl with Load Data Infile in MySQL
	$import->ImportLog($new_log_file);
	$took = $import->thisTook($took);
	
	# Insert the data in the corresponding tables.
	$import->insertIDs();
	
	# Insert the visitor ID's
	$import->insertVisitorIDs();
	
	# Insert the ID's in the main table.
	$import->insertNormalized();
	
	# Add internal referrer detection.
	$import->UpdateInternalReferrers();
	
	# Add useragent detection
	$import->UpdateUseragents();
	
	# Move the crawler traffic to a table of it's own
	$import->separateBots();
	
	# Copy the conversions to a table of it's own
	$import->copyConversions();
	
	# We're done with our log table; delete it.
	$db->Execute("DROP TABLE IF EXISTS {$profile->tablename}log");
	
	if($movedone === true) {
		# If force is true, move the original log file.
		
		if(is_dir($profile->logfilefullpath)) {
			$at = $profile->logfilefullpath.str_replace($profile->logfilefullpath, '', $file);
			$donefile = $profile->logfilefullpath.'done/'.str_replace($profile->logfilefullpath, '', $file);
		} else {
			$at = $file;
			$x = explode("/", $file);
			$fname = $x[count($x) - 1];
			unset($x[count($x) - 1]);
			$donefile = implode("/", $x).'/done/'.$fname;
			unset($x, $fname);
		}
		
		if (rename("{$at}", "{$donefile}")) {
			echoConsoleSafe(_MOVED_FILE_TO." $donefile<P>\n", true);
		} else {
			echo "<span style='color: #F00;'>".str_replace("%s2", $donefile, str_replace("%s1", $at, _MOVING_OF_LOG_FILE_FAILED_DONE_NOT_WRITABLE))."</span>";
		}
	}
	
	# if the $delfile parameter  is true delete the original log file we just processed
	if ($delfile == true) {
		unlink($file);
	}
	
	# delete the temp file we used for importing into the db
	if(file_exists($new_log_file)){
		unlink($new_log_file);
	}
	
	# print some information about this job
	$import->LogProcess("Finished procssing {$human_readable_file}", false);
	$tottime = time() - $start_file_processing;
	$took = $import->thisTook($start_file_processing);
	if ($tottime > 0) {
		$speed = $import->num_import_lines/$tottime;
	} else {
		$speed = $import->num_import_lines;
	}
	$import->LogDuration("Done!", $speed, $tottime);
	$import->LogProcess("Speed was $speed lines per second");
	$import->LogProcess("---------------------------------------------------------", false);
}
$import->LogProcess("Rebuilding index on main table...", false);
$import->EnableKeys($profile->tablename, true);
$import->correctProfileTablename();
$import->EnableKeys($profile->tablename, true);
$took = $import->thisTook($took);



$import->LogProcess(_FINALIZING_UPDATE."...", false);


if ($import->num_import_lines > 0) {
	# Do update summaries for all the data we have just imported.
	include_once("update_summaries.php");
}

# We are done with Perl update; unflag it.
setProfileData($profile->profilename, "{$profile->profilename}.update_running", 'no');
$took = $import->took($st_time);
$totaltook = $import->sec2time($took);
$import->LogProcess("[Finished]".str_replace("%s1", $took, _DONE_UPDATE_TOOK_SECONDS." (".$totaltook.")"), false);
$import->LogProcess("Your profile has been updated.",false);
lgflush();
?>