<?php
set_time_limit(86400);

# These are the files we need to include.
include_once("common.inc.php");
include_once("logparser.inc.php");
include_once("includes/datamanager.php");
include_once("components/import/filehandle.php");
include_once("components/import/import.php");

ob_start();

if(!$running_from_command_line){
	include_once("top.php");
}

lgflush();

# Startup the class
$import = new Import();
$import->manage_keys = true;

# We are running a update, so we set a flag.
setProfileData($profile->profilename, "{$profile->profilename}.update_running", 'yes');

# Open the Update Progress Log, but first check if the dir exists.
if (!file_exists($profile->datamanagerDir.$profile->profilename)) {	
	mkdir($profile->datamanagerDir.$profile->profilename); 
	set_permissions($profile->datamanagerDir.$profile->profilename);
}
$progress_log = $profile->datamanagerDir.$profile->profilename."/{$profile->profilename}_update_progress.lwa.log";
$updatelog = fopen($progress_log, "w");
set_permissions($progress_log);

if(!$php_fallback){
	if (!$running_from_command_line) {
		echoNotice("Your update is running in the background. You don't have to keep this page open.<br/>You can <a href='javascript:showUpdateProgress()'>view update progress</a> here.<br/>Please note that you might experience Logaholic being slower than you're used to, while updating.", 'margin: 5px 10px;');
	} else {
		echo "\nProgress Log = ".trim($progress_log). "\n";
		echoConsoleSafe("Your update is running in php mode...\n ",true);
	}
}
lgflush();

# We want to be able to update while the user is somewhere else, or is gone.
ignore_user_abort(true);

if ($profile->trackermode == 2) {
	if (_LOGAHOLIC_EDITION == 4 || LOGAHOLIC_BASE_EDITION == "cPanel Edition") {
		# not supported in Cpanel
		die("Feature not supported in Cpanel");
	} else if( file_exists("includes/FTP_log_class.php") ){
		require_once 'includes/FTP_log_class.php';
		
		// echoConsoleSafe("<div id='FTP_progress' class='indentbody' style='display: none;'>\n");
		// we have to FTP the file first
		$updir = new UploadDir();
		
		$profile->logfilefullpath = $updir->getUploadDir().$profile->profilename ;
		if(!is_dir($profile->logfilefullpath)){
			mkdir($profile->logfilefullpath);
		}
		
		$profile->logfilefullpath .= '/';
		echoConsoleSafe("Starting FTP download.<br>\n",true);
		
		$FTP_class = new FTP_log($profile->ftpserver, $profile->ftpuser, $profile->ftppasswd, $profile->ftpfullpath, $profile->splitfilter, $profile->splitfilternegative, $profile->logfilefullpath, $import->getStartTimeForProfile());
		$FTP_class->Files_Download();
		echoConsoleSafe("</div><br>\n");
		lgflush();
		
		# if we are doing this we always have to be in multi file mode, so set that now, just in case
		$profile->splitlogs = 1;
		
		# when we've ftp'd we know we don't have to do any more filtering on the files, since that has already been done. So, reset the filters
		$profile->splitfilter = "";
		$profile->splitfilternegative = "";
	}
}

# Check if we have move to done function activated.
if($movedone === true && !file_exists($profile->logfilefullpath.'done/')){
	echoWarning("We are unable to find the '<b>done</b>' directory at the following location:<br/><ul><li>{$profile->logfilefullpath}</li></ul>
	Please create a '<b>done</b>'  directory in your logfile directory <b>({$profile->logfilefullpath})</b><br/>
	We need this to place the completed logfiles in <b>{$profile->logfilefullpath}done/</b>","margin:10px;");
	setProfileData($profile->profilename,"{$profile->profilename}.update_running","no");
	exit();
}


# We want to know how long an update takes.
$st_time = time();

if(empty($mysqltmp)) {
	$mysqltmp = logaholic_dir()."files/";
}

if($profile->trackermode == 1) {
$files[] = "{$mysqltmp}{$profile->profilename}_tracker_log";
// $delfile = true;
} else {
	# Get the log files in array format
	$files = $import->GetLogFilesArray();
}

# lets get started
$import->LogProcess("We're starting an update job. System time is now ".date("Y-m-d H:i:s",time()), false);
$import->LogProcess("Running in PHP mode", false);

$import->correctProfileTablename();

$import->EnableKeys($profile->tablename, false);

$import->totmem = substr(ini_get('memory_limit'), 0, -1);
if (!$import->totmem) {
	$import->totmem = (8 * 1024 * 1024);
} else {
	if ($import->totmem > 14) {
		$fillcache = 1;
	}
	$import->totmem = ($import->totmem * 1024 * 1024);
}
$import->totmem = intval(($import->totmem * 0.9));

	
$skiptime = $import->getStartTimeForProfile();

# Create the regex needed to skip ips.
$regex_skipips = str_replace(" ", "", str_replace(",", "|", $profile->skipips));

# Create the regex needed to skip files.
$skipfiles = str_replace(" ", "", str_replace(",", "|", $profile->skipfiles));
$regex_skipfiles = str_replace('/', '\/', $skipfiles);

$urlparamfiltermode = $profile->urlparamfiltermode;

# set up a clean sessions array
$sessions = array();
$sessioncounter = $import->GetSessionCounter();

# Create the regex needed for urlparams.
if(isset($regex_urlparamfilter)) {
	# Prepare the filter by removing linebreaks and spaces from the filter
	$regex_urlparamfilter = preg_replace("/\n/i", "", $regex_urlparamfilter);
	$regex_urlparamfilter = preg_replace("/ /i", "", $regex_urlparamfilter);

	# Create our filter regex by replaceing commas with pipes
	if($urlparamfiltermode == 'Include') {
		$paramfilter_regex = "";
	} else {
		$paramfilter_regex = $regex_urlparamfilter;
		$paramfilter_regex = preg_replace("/,/i", "|", $paramfilter_regex);
		$tmp_regex = explode('\|', $paramfilter_regex);
		$paramfilter_regex = '(\?|\&)'.implode("=|(\?|\&)", $tmp_regex) . '=';
	}

	$regex_urlparamfilter = '&' . $regex_urlparamfilter . '=';
	$regex_urlparamfilter = preg_replace("/,/i", "=,&", $regex_urlparamfilter);
} else {
	$regex_urlparamfilter = "";
}

# Create the regex for equivdomains
if (!empty($profile->equivdomains)) {
	if(substr($profile->equivdomains, 0, 1) == '(') {
		$regex_equivdomains = $profile->equivdomains;
	} else {
		$equivdomains = str_replace(" ", "", str_replace(",", "|", $profile->equivdomains));
		$equivdomains = explode("|", $equivdomains);
		foreach($equivdomains as $key => $val) {
			$equivdomains[$key] = '://'.$val;
		}
		$regex_equivdomains = implode("|", $equivdomains);
	}
	$regex_equivdomains = str_replace('/', '\/', $regex_equivdomains);
}

# Create the regex for google parameters
$regex_googleparams = str_replace(" ", "", str_replace(",", "|", $profile->googleparams));

# let's keep track of the total number of lines we process during this update run
$totlines=0;

# first, check if we need to run datamanager
$dataManager = new DataManager();
$dataManager->print = 2;
$dataManager->AutoManager($profile);


# For each log file, we'll parse it, and insert it into the database, among other stuff; keep reading to discover more.
foreach($files as $file) {
	if (!file_exists($file)) {
		$import->LogProcess("$file does not exist");
		continue;
	}
	if (filesize($file)==0) {
		$import->LogProcess("$file is empty");
		continue;
	}
	
	# Create a empty array for the bandwidth
	$bandwidth = array();

	# first, check if we need to run datamanager
	$dataManager = new DataManager();
	$dataManager->print = 2;
	$dataManager->AutoManager($profile);
	
	if (substr($profile->logfilefullpath.$file, -3, 3) == ".gz") {
		$filehandle = new gzhandle();
	} else {
		$filehandle = new fhandle();
	}
	
	$logfile = $filehandle->fileopen($file, 'r');
	
	$import->LogProcess("Skiptime is now ".date('Y-m-d',$skiptime));
	
	if (strtoupper(substr(PHP_OS, 0, 3)) !== 'WIN') {
		$load = sys_getloadavg();
		$import->LogProcess("System load {$load[0]} {$load[1]} {$load[2]}");
	}
	
	$start_file_processing = time();
	
	# We set a profile setting so we can keep track how long update is running for the current file.
	setProfileData($profile->profilename, $profile->profilename.".php_preparse_modtime", time());
	
	# This variable is being set to let the user know which file we're currently updating.
	$human_readable_file = explode("/", $file);
	$human_readable_file = $human_readable_file[(count($human_readable_file) - 1)];
	
	# Set the configuration file location
	// $config_file = logaholic_dir()."files/{$profile->profilename}_config";
	
	# Set the preparsed log file location (The file where Perl writes to).
	$new_log_file = "{$mysqltmp}{$profile->profilename}_lwa_log";
	
	# Create the configuration file.
	// $import->WriteConfigFile($config_file, $file, $new_log_file);
	
	$size = round((filesize($file) / 1024) / 1024, 2);
	$format = formatOfLogFile($file);
	$import->LogProcess(_PARSING_LOG_FILE." ({$size} mb, {$format['ClassName']}) ...", false);
	
	$outputfile = fopen("{$mysqltmp}{$profile->profilename}_lwa_log", 'w+');
	set_permissions("{$mysqltmp}{$profile->profilename}_lwa_log");	
	
	if ($import->setLogParser($format['ClassName'])!==false) {
		$import->log_parser->Initialize($logfile);
	} else {
		$import->LogProcess("Unknown log file format");
		continue;
	}
	$c = 0;
	
	# if we have stored a last log position, start reading the file from that position
	$lastlogpos	= getProfileData($profile->profilename, $profile->profilename.".lastlogpos.".md5($file), 0);
	$firstlogline = getProfileData($profile->profilename, $profile->profilename.".firstlogline.".md5($file), 0);
	
	# check if file is still the same 
	$flogline = $filehandle->filegets($logfile, 4096);	
    if ($firstlogline!=md5($flogline)) { 
        $lastlogpos=0;
    }	
	$filehandle->fileseek($logfile, $lastlogpos);
		
	# Loop through each line of the log file
	while ($logline = $filehandle->filegets($logfile, 4096)) {		
		# Convert the log line to our preparsed log format.
		$newline = $import->PHPanalyzeLine($logline);
		
		# Write our newly created log line to our preparsed logfile
		fwrite($outputfile, $newline);
		
		lgflush();
	}
	
	# Fetch the byte value of the file pointer.
	$lastlogpos = $filehandle->filetell($logfile);
	
	fclose($outputfile);
	
	$took = $import->thisTook($start_file_processing);
	
	# If there is no last log position, set it to zero.
	if(!isset($lastlogpos)) {
		$lastlogpos = 0;
	}
	
	# If there is no first log line, make it empty.
	if(!isset($firstlogline)) {
		$firstlogline = '';
	}

	# store the bandwidth in datafiles
	$import->StoreBandwidth($bandwidth);
	
	# Save the first log line and last log position.
	setProfileData($profile->profilename, $profile->profilename.".lastlogpos.".md5($file), $lastlogpos);
	setProfileData($profile->profilename, $profile->profilename.".firstlogline.".md5($file), $firstlogline);
	setProfileData($profile->profilename, $profile->profilename.".lastmodtime.".md5($file), filemtime($file));
	
	# Import the data parsed by perl with Load Data Infile in MySQL
	$import->ImportLog($new_log_file);
	$took = $import->thisTook($took);
	
	# Insert the data in the corresponding tables.
	$import->insertIDs();
	
	# Insert the visitor ID's
	$import->insertVisitorIDs();
	
	# Insert the ID's in the main table.
	$import->insertNormalized();
	
	# Add internal referrer detection.
	$import->UpdateInternalReferrers();
	
	# Add useragent detection
	$import->UpdateUseragents();
	
	# Move the crawler traffic to a table of it's own
	$import->separateBots();
	
	# Copy the conversions to a table of it's own
	$import->copyConversions();
	
	# We're done with our log table; delete it.
	$db->Execute("DROP TABLE IF EXISTS {$profile->tablename}log");	
	
	# We are using the javascript tracker so drop the tracker table.
	if($profile->trackermode == 1) {
		$db->Execute("DELETE FROM {$profile->tablename_trackerlog}");
	}
	
	if($movedone === true) {
		# If force is true, move the original log file.
		
		if(is_dir($profile->logfilefullpath)) {
			$at = $profile->logfilefullpath.str_replace($profile->logfilefullpath, '', $file);
			$donefile = $profile->logfilefullpath.'done/'.str_replace($profile->logfilefullpath, '', $file);
		} else {
			$at = $file;
			$x = explode("/", $file);
			$fname = $x[count($x) - 1];
			unset($x[count($x) - 1]);
			$donefile = implode("/", $x).'/done/'.$fname;
			unset($x, $fname);
		}
		
		if (rename("{$at}", "{$donefile}")) {
			echoConsoleSafe(_MOVED_FILE_TO." $donefile<P>\n", true);
		} else {
			echo "<span style='color: #F00;'>".str_replace("%s2", $donefile, str_replace("%s1", $at, _MOVING_OF_LOG_FILE_FAILED_DONE_NOT_WRITABLE))."</span>";
		}
	}
	
	# Remove the tracker log file..
	if($profile->trackermode == 1) {
		if(!@unlink("{$mysqltmp}{$profile->profilename}_tracker_log")){
			echoWarning("Failed to delete file: <b>$file</b>","margin:5px 10px;");
		}
	}
	
	# if the $delfile parameter  is true delete the original log file we just processed
	if ($delfile == true) {
		if(!@unlink($file)){
			echoWarning("Failed to delete file: <b>$file</b>","margin:5px 10px;");
		}
	}
	
	# delete the temp file we used for importing into the db
	if(file_exists($new_log_file)){
		if(!@unlink($new_log_file)){
			echoWarning("Failed to delete file: <b>$file</b>","margin:5px 10px;");
		}
	}
	
	# print some information about this job
	$import->LogProcess("Finished processing {$human_readable_file}", false);
	$tottime = time() - $start_file_processing;
	$took = $import->thisTook($start_file_processing);
	if ($tottime > 0) {
		$speed = $import->num_import_lines / $tottime;
	} else {
		$speed = $import->num_import_lines;
	}
	$totlines= $totlines + $import->num_import_lines;
	$import->LogDuration("Done!", $speed, $tottime);
	$import->LogProcess("Imported {$import->num_import_lines}, Speed was $speed lines per second");
	$import->LogProcess("---------------------------------------------------------", false);
	# update the sessioncounter
	setProfileData($profile->profilename, "$profile->profilename.sessioncounter", $sessioncounter);
	
	#close the filehandler
	$filehandle->fileclose($logfile);
}
$import->LogProcess("Rebuilding index on main table...", false);
$import->EnableKeys($profile->tablename, true);
$import->correctProfileTablename();
$import->EnableKeys($profile->tablename, true);
$took = $import->thisTook($took);

$import->LogProcess(_FINALIZING_UPDATE."...", false);

if ($totlines > 0) {
	// echo "Do Update summraies!<br/>";
	# Do update summaries for all the data we have just imported.
	include_once("update_summaries.php");
	// echo "Done Update summraies!";
}
# We are done with update; unflag it.
setProfileData($profile->profilename, "{$profile->profilename}.update_running", 'no');
$took = $import->took($st_time);
$totaltook = $import->sec2time($took);
if ($took > 0) {
	$lps = round($totlines/$took);
} else {
	$lps = $totlines;
}
$lastline = "[Finished] Processed $totlines lines ($lps/lps). ".str_replace("%s1", $took, _DONE_UPDATE_TOOK_SECONDS." (".$totaltook.")");
$import->LogProcess($lastline, false);
echo $lastline."\n";
lgflush();

fclose($updatelog);

$updatelog = '';
?>